@@ -533,6 +533,7 @@ UINT WINAPI ImeToAsciiEx( UINT vkey, UINT vsc, BYTE *state, TRANSMSGLIST *msgs,
     UINT size, count = 0;
     INPUTCONTEXT *ctx;
     NTSTATUS status;
+    BOOL key_consumed = TRUE;
 
     TRACE( "vkey %#x, vsc %#x, state %p, msgs %p, flags %#x, himc %p\n",
            vkey, vsc, state, msgs, flags, himc );
@@ -551,6 +552,7 @@ UINT WINAPI ImeToAsciiEx( UINT vkey, UINT vsc, BYTE *state, TRANSMSGLIST *msgs,
         if (!(compstr = ImmLockIMCC( (ctx->hCompStr = himcc) ))) goto done;
 
         params.compstr = compstr;
+        params.key_consumed = &key_consumed;
         status = NtUserMessageCall( ctx->hWnd, WINE_IME_TO_ASCII_EX, vkey, vsc, &params,
                                     NtUserImeDriverCall, FALSE );
         size = compstr->dwSize;
@@ -559,14 +561,17 @@ UINT WINAPI ImeToAsciiEx( UINT vkey, UINT vsc, BYTE *state, TRANSMSGLIST *msgs,
     if (status) WARN( "WINE_IME_TO_ASCII_EX returned status %#lx\n", status );
     else
     {
-        TRANSMSG status_msg = {.message = ime_set_composition_status( himc, !!compstr->dwCompStrOffset )};
-        if (status_msg.message) msgs->TransMsg[count++] = status_msg;
+        if (compstr->dwCompStrOffset || compstr->dwResultStrLen)
+        {
+            TRANSMSG msg = {.message = ime_set_composition_status( himc, TRUE )};
+            if (msg.message == WM_IME_STARTCOMPOSITION) msgs->TransMsg[count++] = msg;
+        }
 
-        if (compstr->dwResultStrOffset)
+        if (compstr->dwResultStrLen)
         {
             const WCHAR *result = (WCHAR *)((BYTE *)compstr + compstr->dwResultStrOffset);
             TRANSMSG msg = {.message = WM_IME_COMPOSITION, .wParam = result[0], .lParam = GCS_RESULTSTR};
-            if (compstr->dwResultClauseOffset) msg.lParam |= GCS_RESULTCLAUSE;
+            if (compstr->dwResultClauseLen) msg.lParam |= GCS_RESULTCLAUSE;
             msgs->TransMsg[count++] = msg;
         }
 
@@ -579,6 +584,18 @@ UINT WINAPI ImeToAsciiEx( UINT vkey, UINT vsc, BYTE *state, TRANSMSGLIST *msgs,
             else msg.lParam |= CS_INSERTCHAR|CS_NOMOVECARET;
             msgs->TransMsg[count++] = msg;
         }
+
+        if (!compstr->dwCompStrLen)
+        {
+            TRANSMSG msg = {.message = ime_set_composition_status( himc, FALSE )};
+            if (msg.message == WM_IME_ENDCOMPOSITION) msgs->TransMsg[count++] = msg;
+        }
+
+        if (!key_consumed)
+        {
+            TRANSMSG msg = {.message = WM_IME_KEYDOWN, .wParam = vkey, .lParam = vsc};
+            msgs->TransMsg[count++] = msg;
+        }
     }
 
     ImmUnlockIMCC( ctx->hCompStr );
