@@ -1501,21 +1501,23 @@ static void String_destructor(jsdisp_t *dispex)
     jsstr_release(This->str);
 }
 
-static unsigned String_idx_length(jsdisp_t *jsdisp)
+static HRESULT String_lookup_prop(jsdisp_t *jsdisp, const WCHAR *name, unsigned flags, struct property_info *desc)
 {
     StringInstance *string = string_from_jsdisp(jsdisp);
 
     /*
      * NOTE: For invoke version < 2, indexed array is not implemented at all.
      * Newer jscript.dll versions implement it on string type, not class,
-     * which is not how it should work according to spec. IE9 implements it
-     * properly, but it uses its own JavaScript engine inside MSHTML. We
-     * implement it here, but in the way IE9 and spec work.
+     * which is not how it should work according to spec. IE9+ implements it
+     * properly.
      */
-    return string->dispex.ctx->version < 2 ? 0 : jsstr_length(string->str);
+    if(string->dispex.ctx->version < 2)
+        return DISP_E_UNKNOWNNAME;
+
+    return jsdisp_index_lookup(&string->dispex, name, jsstr_length(string->str), desc);
 }
 
-static HRESULT String_idx_get(jsdisp_t *jsdisp, unsigned idx, jsval_t *r)
+static HRESULT String_prop_get(jsdisp_t *jsdisp, unsigned idx, jsval_t *r)
 {
     StringInstance *string = string_from_jsdisp(jsdisp);
     jsstr_t *ret;
@@ -1530,6 +1532,16 @@ static HRESULT String_idx_get(jsdisp_t *jsdisp, unsigned idx, jsval_t *r)
     return S_OK;
 }
 
+static HRESULT String_fill_props(jsdisp_t *jsdisp)
+{
+    StringInstance *string = string_from_jsdisp(jsdisp);
+
+    if(string->dispex.ctx->version < 2)
+        return S_OK;
+
+    return jsdisp_fill_indices(&string->dispex, jsstr_length(string->str));
+}
+
 static const builtin_prop_t String_props[] = {
     {L"anchor",                String_anchor,                PROPF_METHOD|1},
     {L"big",                   String_big,                   PROPF_METHOD},
@@ -1579,12 +1591,13 @@ static const builtin_prop_t StringInst_props[] = {
 };
 
 static const builtin_info_t StringInst_info = {
-    .class      = JSCLASS_STRING,
-    .props_cnt  = ARRAY_SIZE(StringInst_props),
-    .props      = StringInst_props,
-    .destructor = String_destructor,
-    .idx_length = String_idx_length,
-    .idx_get    = String_idx_get,
+    .class       = JSCLASS_STRING,
+    .props_cnt   = ARRAY_SIZE(StringInst_props),
+    .props       = StringInst_props,
+    .destructor  = String_destructor,
+    .lookup_prop = String_lookup_prop,
+    .prop_get    = String_prop_get,
+    .fill_props  = String_fill_props,
 };
 
 /* ECMA-262 3rd Edition    15.5.3.2 */
