@@ -122,13 +122,12 @@ static const event_target_vtbl_t HTMLTitleElement_event_target_vtbl = {
 };
 
 static const tid_t HTMLTitleElement_iface_tids[] = {
-    HTMLELEMENT_TIDS,
     IHTMLTitleElement_tid,
     0
 };
 dispex_static_data_t HTMLTitleElement_dispex = {
-    .id           = PROT_HTMLTitleElement,
-    .prototype_id = PROT_HTMLElement,
+    .id           = OBJID_HTMLTitleElement,
+    .prototype_id = OBJID_HTMLElement,
     .vtbl         = &HTMLTitleElement_event_target_vtbl.dispex_vtbl,
     .disp_tid     = DispHTMLTitleElement_tid,
     .iface_tids   = HTMLTitleElement_iface_tids,
@@ -238,13 +237,12 @@ static const event_target_vtbl_t HTMLHtmlElement_event_target_vtbl = {
 };
 
 static const tid_t HTMLHtmlElement_iface_tids[] = {
-    HTMLELEMENT_TIDS,
     IHTMLHtmlElement_tid,
     0
 };
 dispex_static_data_t HTMLHtmlElement_dispex = {
-    .id           = PROT_HTMLHtmlElement,
-    .prototype_id = PROT_HTMLElement,
+    .id           = OBJID_HTMLHtmlElement,
+    .prototype_id = OBJID_HTMLElement,
     .vtbl         = &HTMLHtmlElement_event_target_vtbl.dispex_vtbl,
     .disp_tid     = DispHTMLHtmlElement_tid,
     .iface_tids   = HTMLHtmlElement_iface_tids,
@@ -417,14 +415,13 @@ static const event_target_vtbl_t HTMLMetaElement_event_target_vtbl = {
 };
 
 static const tid_t HTMLMetaElement_iface_tids[] = {
-    HTMLELEMENT_TIDS,
     IHTMLMetaElement_tid,
     0
 };
 
 dispex_static_data_t HTMLMetaElement_dispex = {
-    .id           = PROT_HTMLMetaElement,
-    .prototype_id = PROT_HTMLElement,
+    .id           = OBJID_HTMLMetaElement,
+    .prototype_id = OBJID_HTMLElement,
     .vtbl         = &HTMLMetaElement_event_target_vtbl.dispex_vtbl,
     .disp_tid     = DispHTMLMetaElement_tid,
     .iface_tids   = HTMLMetaElement_iface_tids,
@@ -525,13 +522,12 @@ static const event_target_vtbl_t HTMLHeadElement_event_target_vtbl = {
 };
 
 static const tid_t HTMLHeadElement_iface_tids[] = {
-    HTMLELEMENT_TIDS,
     IHTMLHeadElement_tid,
     0
 };
 dispex_static_data_t HTMLHeadElement_dispex = {
-    .id           = PROT_HTMLHeadElement,
-    .prototype_id = PROT_HTMLElement,
+    .id           = OBJID_HTMLHeadElement,
+    .prototype_id = OBJID_HTMLElement,
     .vtbl         = &HTMLHeadElement_event_target_vtbl.dispex_vtbl,
     .disp_tid     = DispHTMLHeadElement_tid,
     .iface_tids   = HTMLHeadElement_iface_tids,
