@@ -1621,23 +1621,6 @@ int CDECL _isnan(double num)
 
 #if _MSVCR_VER>=120
 
-/*********************************************************************
- *      rint (MSVCR120.@)
- */
-double CDECL MSVCRT_rint(double x)
-{
-    unsigned cw;
-    double y;
-
-    cw = _controlfp(0, 0);
-    if ((cw & _MCW_PC) != _PC_53)
-        _controlfp(_PC_53, _MCW_PC);
-    y = rint(x);
-    if ((cw & _MCW_PC) != _PC_53)
-        _controlfp(cw, _MCW_PC);
-    return y;
-}
-
 /*********************************************************************
  *		_nearbyint (MSVCR120.@)
  *
@@ -1656,7 +1639,7 @@ double CDECL nearbyint(double x)
         cw |= _EM_INEXACT;
         _setfp(&cw, _EM_INEXACT, NULL, 0);
     }
-    x = MSVCRT_rint(x);
+    x = rint(x);
     if (update_cw || update_sw)
     {
         sw = 0;
@@ -2603,7 +2586,7 @@ __msvcrt_long CDECL lrint(double x)
 {
     double d;
 
-    d = MSVCRT_rint(x);
+    d = rint(x);
     if ((d < 0 && d != (double)(__msvcrt_long)d)
             || (d >= 0 && d != (double)(__msvcrt_ulong)d)) {
         *_errno() = EDOM;
@@ -2635,7 +2618,7 @@ __int64 CDECL llrint(double x)
 {
     double d;
 
-    d = MSVCRT_rint(x);
+    d = rint(x);
     if ((d < 0 && d != (double)(__int64)d)
             || (d >= 0 && d != (double)(unsigned __int64)d)) {
         *_errno() = EDOM;
