@@ -177,7 +177,8 @@ struct syscall_frame
     ULONG                 restore_flags;  /* 10c */
     struct syscall_frame *prev_frame;     /* 110 */
     void                 *syscall_cfa;    /* 118 */
-    ULONG64               align;          /* 120 */
+    ULONG                 syscall_id;     /* 120 */
+    ULONG                 align;          /* 124 */
     ULONG                 fpcr;           /* 128 */
     ULONG                 fpsr;           /* 12c */
     NEON128               v[32];          /* 130 */
@@ -185,26 +186,6 @@ struct syscall_frame
 
 C_ASSERT( sizeof( struct syscall_frame ) == 0x330 );
 
-struct arm64_thread_data
-{
-    struct syscall_frame *syscall_frame; /* 02f0 frame pointer on syscall entry */
-    SYSTEM_SERVICE_TABLE *syscall_table; /* 02f8 syscall table */
-};
-
-C_ASSERT( sizeof(struct arm64_thread_data) <= sizeof(((struct ntdll_thread_data *)0)->cpu_data) );
-C_ASSERT( offsetof( TEB, GdiTebBatch ) + offsetof( struct arm64_thread_data, syscall_frame ) == 0x2f0 );
-C_ASSERT( offsetof( TEB, GdiTebBatch ) + offsetof( struct arm64_thread_data, syscall_table ) == 0x2f8 );
-
-static inline struct arm64_thread_data *arm64_thread_data(void)
-{
-    return (struct arm64_thread_data *)ntdll_get_thread_data()->cpu_data;
-}
-
-static BOOL is_inside_syscall( ucontext_t *sigcontext )
-{
-    return ((char *)SP_sig(sigcontext) >= (char *)ntdll_get_thread_data()->kernel_stack &&
-            (char *)SP_sig(sigcontext) <= (char *)arm64_thread_data()->syscall_frame);
-}
 
 /***********************************************************************
  *           context_init_empty_xstate
@@ -335,7 +316,7 @@ static void restore_context( const CONTEXT *context, ucontext_t *sigcontext )
  */
 NTSTATUS signal_set_full_context( CONTEXT *context )
 {
-    struct syscall_frame *frame = arm64_thread_data()->syscall_frame;
+    struct syscall_frame *frame = get_syscall_frame();
     NTSTATUS status = NtSetContextThread( GetCurrentThread(), context );
 
     if (!status && (context->ContextFlags & CONTEXT_INTEGER) == CONTEXT_INTEGER)
@@ -378,7 +359,7 @@ void *get_wow_context( CONTEXT *context )
  */
 NTSTATUS WINAPI NtSetContextThread( HANDLE handle, const CONTEXT *context )
 {
-    struct syscall_frame *frame = arm64_thread_data()->syscall_frame;
+    struct syscall_frame *frame = get_syscall_frame();
     NTSTATUS ret = STATUS_SUCCESS;
     BOOL self = (handle == GetCurrentThread());
     DWORD flags = context->ContextFlags & ~CONTEXT_ARM64;
@@ -427,7 +408,7 @@ NTSTATUS WINAPI NtSetContextThread( HANDLE handle, const CONTEXT *context )
  */
 NTSTATUS WINAPI NtGetContextThread( HANDLE handle, CONTEXT *context )
 {
-    struct syscall_frame *frame = arm64_thread_data()->syscall_frame;
+    struct syscall_frame *frame = get_syscall_frame();
     DWORD needed_flags = context->ContextFlags & ~CONTEXT_ARM64;
     BOOL self = (handle == GetCurrentThread());
 
@@ -769,7 +750,7 @@ static void setup_exception( ucontext_t *sigcontext, EXCEPTION_RECORD *rec )
 NTSTATUS call_user_apc_dispatcher( CONTEXT *context, ULONG_PTR arg1, ULONG_PTR arg2, ULONG_PTR arg3,
                                    PNTAPCFUNC func, NTSTATUS status )
 {
-    struct syscall_frame *frame = arm64_thread_data()->syscall_frame;
+    struct syscall_frame *frame = get_syscall_frame();
     ULONG64 sp = context ? context->Sp : frame->sp;
     struct apc_stack_layout *stack;
 
@@ -805,7 +786,7 @@ NTSTATUS call_user_apc_dispatcher( CONTEXT *context, ULONG_PTR arg1, ULONG_PTR a
  */
 void call_raise_user_exception_dispatcher(void)
 {
-    arm64_thread_data()->syscall_frame->pc = (UINT64)pKiRaiseUserExceptionDispatcher;
+    get_syscall_frame()->pc = (UINT64)pKiRaiseUserExceptionDispatcher;
 }
 
 
@@ -814,7 +795,7 @@ void call_raise_user_exception_dispatcher(void)
  */
 NTSTATUS call_user_exception_dispatcher( EXCEPTION_RECORD *rec, CONTEXT *context )
 {
-    struct syscall_frame *frame = arm64_thread_data()->syscall_frame;
+    struct syscall_frame *frame = get_syscall_frame();
     struct exc_stack_layout *stack;
     NTSTATUS status = NtSetContextThread( GetCurrentThread(), context );
 
@@ -840,10 +821,10 @@ NTSTATUS call_user_exception_dispatcher( EXCEPTION_RECORD *rec, CONTEXT *context
 extern NTSTATUS call_user_mode_callback( ULONG64 user_sp, void **ret_ptr, ULONG *ret_len,
                                          void *func, TEB *teb );
 __ASM_GLOBAL_FUNC( call_user_mode_callback,
-                   "stp x29, x30, [sp,#-0xc0]!\n\t"
-                   __ASM_CFI(".cfi_def_cfa_offset 0xc0\n\t")
-                   __ASM_CFI(".cfi_offset 29,-0xc0\n\t")
-                   __ASM_CFI(".cfi_offset 30,-0xb8\n\t")
+                   "stp x29, x30, [sp,#-0xd0]!\n\t"
+                   __ASM_CFI(".cfi_def_cfa_offset 0xd0\n\t")
+                   __ASM_CFI(".cfi_offset 29,-0xd0\n\t")
+                   __ASM_CFI(".cfi_offset 30,-0xc8\n\t")
                    "mov x29, sp\n\t"
                    __ASM_CFI(".cfi_def_cfa_register 29\n\t")
                    "stp x19, x20, [x29, #0x10]\n\t"
@@ -873,14 +854,27 @@ __ASM_GLOBAL_FUNC( call_user_mode_callback,
                    "ldr x2, [x18]\n\t"            /* teb->Tib.ExceptionList */
                    "stp x1, x2, [x29, #0xb0]\n\t"
 
-                   "ldr x7, [x18, #0x2f0]\n\t"    /* arm64_thread_data()->syscall_frame */
+                   "ldr x7, [x18, #0x378]\n\t"    /* thread_data->syscall_frame */
                    "sub x1, sp, #0x330\n\t"       /* sizeof(struct syscall_frame) */
-                   "str x1, [x18, #0x2f0]\n\t"    /* arm64_thread_data()->syscall_frame */
-                   "add x8, x29, #0xc0\n\t"
+                   "str x1, [x18, #0x378]\n\t"    /* thread_data->syscall_frame */
+                   "add x8, x29, #0xd0\n\t"
                    "stp x7, x8, [x1, #0x110]\n\t" /* frame->prev_frame,syscall_cfa */
+                   "ldr w11, [x18, #0x380]\n\t"   /* thread_data->syscall_trace */
+                   "cbnz x11, 1f\n\t"
                    /* switch to user stack */
                    "mov sp, x0\n\t"               /* user_sp */
-                   "br x3" )
+                   "br x3\n"
+                   "1:\tmov x19, x18\n\t"         /* teb */
+                   "mov x20, x0\n\t"              /* user_sp */
+                   "mov x21, x3\n\t"              /* func */
+                   "mov sp, x1\n\t"
+                   "ldr x1, [x20]\n\t"            /* args */
+                   "ldp w2, w0, [x20, #8]\n\t"    /* len, id */
+                   "str x0, [x29, #0xc0]\n\t"     /* id */
+                   "bl " __ASM_NAME("trace_usercall") "\n\t"
+                   "mov x18, x19\n\t"             /* teb */
+                   "mov sp, x20\n\t"              /* user_sp */
+                   "br x21" )
 
 
 /***********************************************************************
@@ -889,10 +883,10 @@ __ASM_GLOBAL_FUNC( call_user_mode_callback,
 extern void DECLSPEC_NORETURN user_mode_callback_return( void *ret_ptr, ULONG ret_len,
                                                          NTSTATUS status, TEB *teb );
 __ASM_GLOBAL_FUNC( user_mode_callback_return,
-                   "ldr x4, [x3, #0x2f0]\n\t"     /* arm64_thread_data()->syscall_frame */
+                   "ldr x4, [x3, #0x378]\n\t"     /* thread_data->syscall_frame */
                    "ldp x5, x29, [x4,#0x110]\n\t" /* prev_frame,syscall_cfa */
-                   "str x5, [x3, #0x2f0]\n\t"     /* arm64_thread_data()->syscall_frame */
-                   "sub x29, x29, #0xc0\n\t"
+                   "str x5, [x3, #0x378]\n\t"     /* thread_data->syscall_frame */
+                   "sub x29, x29, #0xd0\n\t"
                    __ASM_CFI(".cfi_def_cfa_register 29\n\t")
                    __ASM_CFI(".cfi_rel_offset 29,0x00\n\t")
                    __ASM_CFI(".cfi_rel_offset 30,0x08\n\t")
@@ -911,7 +905,16 @@ __ASM_GLOBAL_FUNC( user_mode_callback_return,
                    "msr fpcr, x5\n\t"
                    "lsr x5, x5, #32\n\t"
                    "msr fpsr, x5\n\t"
-                   "ldp x19, x20, [x29, #0x10]\n\t"
+                   "ldp x5, x6, [x29, #0xa0]\n\t" /* ret_ptr, ret_len */
+                   "str x0, [x5]\n\t"             /* ret_ptr */
+                   "str w1, [x6]\n\t"             /* ret_len */
+                   "ldr w11, [x3, #0x380]\n\t"    /* thread_data->syscall_trace */
+                   "cbz x11, 1f\n\t"
+                   "ldr w3, [x29, #0xc0]\n\t"     /* id */
+                   "mov x19, x2\n\t"
+                   "bl " __ASM_NAME("trace_userret") "\n\t"
+                   "mov x2, x19\n"                /* status */
+                   "1:\tldp x19, x20, [x29, #0x10]\n\t"
                    __ASM_CFI(".cfi_same_value 19\n\t")
                    __ASM_CFI(".cfi_same_value 20\n\t")
                    "ldp x21, x22, [x29, #0x20]\n\t"
@@ -930,12 +933,9 @@ __ASM_GLOBAL_FUNC( user_mode_callback_return,
                    "ldp d10, d11, [x29, #0x70]\n\t"
                    "ldp d12, d13, [x29, #0x80]\n\t"
                    "ldp d14, d15, [x29, #0x90]\n\t"
-                   "ldp x5, x6, [x29, #0xa0]\n\t" /* ret_ptr, ret_len */
-                   "str x0, [x5]\n\t"             /* ret_ptr */
-                   "str w1, [x6]\n\t"             /* ret_len */
                    "mov x0, x2\n\t"               /* status */
                    "mov sp, x29\n\t"
-                   "ldp x29, x30, [sp], #0xc0\n\t"
+                   "ldp x29, x30, [sp], #0xd0\n\t"
                    "ret" )
 
 
@@ -969,7 +969,7 @@ __ASM_GLOBAL_FUNC( user_mode_abort_thread,
  */
 NTSTATUS KeUserModeCallback( ULONG id, const void *args, ULONG len, void **ret_ptr, ULONG *ret_len )
 {
-    struct syscall_frame *frame = arm64_thread_data()->syscall_frame;
+    struct syscall_frame *frame = get_syscall_frame();
     ULONG64 sp = (frame->sp - offsetof( struct callback_stack_layout, args_data[len] ) - 16) & ~15;
     struct callback_stack_layout *stack = (struct callback_stack_layout *)sp;
 
@@ -992,7 +992,7 @@ NTSTATUS KeUserModeCallback( ULONG id, const void *args, ULONG len, void **ret_p
  */
 NTSTATUS WINAPI NtCallbackReturn( void *ret_ptr, ULONG ret_len, NTSTATUS status )
 {
-    if (!arm64_thread_data()->syscall_frame->prev_frame) return STATUS_NO_CALLBACK_ACTIVE;
+    if (!get_syscall_frame()->prev_frame) return STATUS_NO_CALLBACK_ACTIVE;
     user_mode_callback_return( ret_ptr, ret_len, status, NtCurrentTeb() );
 }
 
@@ -1004,10 +1004,10 @@ NTSTATUS WINAPI NtCallbackReturn( void *ret_ptr, ULONG ret_len, NTSTATUS status
  */
 static BOOL handle_syscall_fault( ucontext_t *context, EXCEPTION_RECORD *rec )
 {
-    struct syscall_frame *frame = arm64_thread_data()->syscall_frame;
+    struct syscall_frame *frame = get_syscall_frame();
     DWORD i;
 
-    if (!is_inside_syscall( context )) return FALSE;
+    if (!is_inside_syscall( SP_sig(context) )) return FALSE;
 
     TRACE( "code=%x flags=%x addr=%p pc=%p tid=%04x\n",
            rec->ExceptionCode, rec->ExceptionFlags, rec->ExceptionAddress,
@@ -1051,9 +1051,10 @@ static BOOL handle_syscall_fault( ucontext_t *context, EXCEPTION_RECORD *rec )
     else
     {
         TRACE( "returning to user mode ip=%p ret=%08x\n", (void *)frame->pc, rec->ExceptionCode );
-        REGn_sig(0, context) = (ULONG_PTR)frame;
-        REGn_sig(1, context) = rec->ExceptionCode;
-        PC_sig(context)      = (ULONG_PTR)__wine_syscall_dispatcher_return;
+        REGn_sig(0, context)  = rec->ExceptionCode;
+        REGn_sig(18, context) = (ULONG_PTR)NtCurrentTeb();
+        SP_sig(context)       = (ULONG_PTR)frame;
+        PC_sig(context)       = (ULONG_PTR)__wine_syscall_dispatcher_return;
     }
     return TRUE;
 }
@@ -1068,9 +1069,12 @@ static void segv_handler( int signal, siginfo_t *siginfo, void *sigcontext )
 {
     EXCEPTION_RECORD rec = { 0 };
     ucontext_t *context = sigcontext;
+    DWORD64 esr = get_fault_esr( context );
 
     rec.NumberParameters = 2;
-    rec.ExceptionInformation[0] = (get_fault_esr( context ) & 0x40) != 0;
+    if ((esr & 0xf0000000) == 0x80000000) rec.ExceptionInformation[0] = EXCEPTION_EXECUTE_FAULT;
+    else if (esr & 0x40) rec.ExceptionInformation[0] = EXCEPTION_WRITE_FAULT;
+    else rec.ExceptionInformation[0] = EXCEPTION_READ_FAULT;
     rec.ExceptionInformation[1] = (ULONG_PTR)siginfo->si_addr;
     if (!virtual_handle_fault( &rec, (void *)SP_sig(context) )) return;
     if (handle_syscall_fault( context, &rec )) return;
@@ -1271,7 +1275,9 @@ static void abrt_handler( int signal, siginfo_t *siginfo, void *sigcontext )
  */
 static void quit_handler( int signal, siginfo_t *siginfo, void *sigcontext )
 {
-    if (!is_inside_syscall( sigcontext )) user_mode_abort_thread( 0, arm64_thread_data()->syscall_frame );
+    ucontext_t *context = sigcontext;
+
+    if (!is_inside_syscall( SP_sig(context) )) user_mode_abort_thread( 0, get_syscall_frame() );
     abort_thread(0);
 }
 
@@ -1283,9 +1289,10 @@ static void quit_handler( int signal, siginfo_t *siginfo, void *sigcontext )
  */
 static void usr1_handler( int signal, siginfo_t *siginfo, void *sigcontext )
 {
+    ucontext_t *ucontext = sigcontext;
     CONTEXT context;
 
-    if (is_inside_syscall( sigcontext ))
+    if (is_inside_syscall( SP_sig(ucontext) ))
     {
         context.ContextFlags = CONTEXT_FULL | CONTEXT_EXCEPTION_REQUEST;
         NtGetContextThread( GetCurrentThread(), &context );
@@ -1294,10 +1301,10 @@ static void usr1_handler( int signal, siginfo_t *siginfo, void *sigcontext )
     }
     else
     {
-        save_context( &context, sigcontext );
+        save_context( &context, ucontext );
         context.ContextFlags |= CONTEXT_EXCEPTION_REPORTING;
         wait_suspend( &context );
-        restore_context( &context, sigcontext );
+        restore_context( &context, ucontext );
     }
 }
 
@@ -1309,11 +1316,11 @@ static void usr1_handler( int signal, siginfo_t *siginfo, void *sigcontext )
  */
 static void usr2_handler( int signal, siginfo_t *siginfo, void *sigcontext )
 {
-    struct syscall_frame *frame = arm64_thread_data()->syscall_frame;
+    struct syscall_frame *frame = get_syscall_frame();
     ucontext_t *context = sigcontext;
     DWORD i;
 
-    if (!is_inside_syscall( sigcontext )) return;
+    if (!is_inside_syscall( SP_sig(context) )) return;
 
     FP_sig(context)     = frame->fp;
     LR_sig(context)     = frame->lr;
@@ -1390,9 +1397,10 @@ void signal_free_thread( TEB *teb )
 void signal_init_process(void)
 {
     struct sigaction sig_act;
-    void *kernel_stack = (char *)ntdll_get_thread_data()->kernel_stack + kernel_stack_size;
+    struct ntdll_thread_data *thread_data = ntdll_get_thread_data();
+    void *kernel_stack = (char *)thread_data->kernel_stack + kernel_stack_size;
 
-    arm64_thread_data()->syscall_frame = (struct syscall_frame *)kernel_stack - 1;
+    thread_data->syscall_frame = (struct syscall_frame *)kernel_stack - 1;
 
     sig_act.sa_mask = server_block_set;
     sig_act.sa_flags = SA_SIGINFO | SA_RESTART | SA_ONSTACK;
@@ -1434,18 +1442,15 @@ void syscall_dispatcher_return_slowpath(void)
 }
 
 /***********************************************************************
- *           call_init_thunk
+ *           init_syscall_frame
  */
-void call_init_thunk( LPTHREAD_START_ROUTINE entry, void *arg, BOOL suspend, TEB *teb,
-                      struct syscall_frame *frame, void *syscall_cfa )
+void init_syscall_frame( LPTHREAD_START_ROUTINE entry, void *arg, BOOL suspend, TEB *teb )
 {
-    struct arm64_thread_data *thread_data = (struct arm64_thread_data *)&teb->GdiTebBatch;
+    struct syscall_frame *frame = ((struct ntdll_thread_data *)&teb->GdiTebBatch)->syscall_frame;
     CONTEXT *ctx, context = { CONTEXT_ALL };
     I386_CONTEXT *i386_context;
     ARM_CONTEXT *arm_context;
 
-    thread_data->syscall_table = KeServiceDescriptorTable;
-
     context.X0  = (DWORD64)entry;
     context.X1  = (DWORD64)arg;
     context.X18 = (DWORD64)teb;
@@ -1490,19 +1495,15 @@ void call_init_thunk( LPTHREAD_START_ROUTINE entry, void *arg, BOOL suspend, TEB
     ctx = (CONTEXT *)((ULONG_PTR)context.Sp & ~15) - 1;
     *ctx = context;
     ctx->ContextFlags = CONTEXT_FULL;
-    memset( frame, 0, sizeof(*frame) );
-    NtSetContextThread( GetCurrentThread(), ctx );
+    signal_set_full_context( ctx );
 
     frame->sp    = (ULONG64)ctx;
     frame->pc    = (ULONG64)pLdrInitializeThunk;
     frame->x[0]  = (ULONG64)ctx;
     frame->x[18] = (ULONG64)teb;
-    frame->restore_flags |= CONTEXT_INTEGER;
-    frame->syscall_cfa    = syscall_cfa;
     syscall_frame_fixup_for_fastpath( frame );
 
     pthread_sigmask( SIG_UNBLOCK, &server_block_set, NULL );
-    __wine_syscall_dispatcher_return( frame, 0 );
 }
 
 
@@ -1533,20 +1534,23 @@ __ASM_GLOBAL_FUNC( signal_start_thread,
                    __ASM_CFI(".cfi_rel_offset 28,0x58\n\t")
                    "add x5, x29, #0xc0\n\t"     /* syscall_cfa */
                    /* set syscall frame */
-                   "ldr x4, [x3, #0x2f0]\n\t"   /* arm64_thread_data()->syscall_frame */
+                   "ldr x4, [x3, #0x378]\n\t"   /* thread_data->syscall_frame */
                    "cbnz x4, 1f\n\t"
                    "sub x4, sp, #0x330\n\t"     /* sizeof(struct syscall_frame) */
-                   "str x4, [x3, #0x2f0]\n\t"   /* arm64_thread_data()->syscall_frame */
+                   "str x4, [x3, #0x378]\n\t"   /* thread_data->syscall_frame */
+                   "1:\tstr wzr, [x4, #0x10c]\n\t" /* frame->restore_flags */
+                   "stp xzr, x5, [x4, #0x110]\n\t" /* frame->prev_frame,syscall_cfa */
                    /* switch to kernel stack */
-                   "1:\tmov sp, x4\n\t"
-                   "bl " __ASM_NAME("call_init_thunk") )
+                   "mov sp, x4\n\t"
+                   "bl " __ASM_NAME("init_syscall_frame") "\n\t"
+                   "b " __ASM_LOCAL_LABEL("__wine_syscall_dispatcher_return") )
 
 
 /***********************************************************************
  *           __wine_syscall_dispatcher
  */
 __ASM_GLOBAL_FUNC( __wine_syscall_dispatcher,
-                   "ldr x10, [x18, #0x2f0]\n\t" /* arm64_thread_data()->syscall_frame */
+                   "ldr x10, [x18, #0x378]\n\t" /* thread_data->syscall_frame */
                    "stp x18, x19, [x10, #0x90]\n\t"
                    "stp x20, x21, [x10, #0xa0]\n\t"
                    "stp x22, x23, [x10, #0xb0]\n\t"
@@ -1557,6 +1561,7 @@ __ASM_GLOBAL_FUNC( __wine_syscall_dispatcher,
                    "stp x9, x19, [x10, #0xf0]\n\t"
                    "mrs x9, NZCV\n\t"
                    "stp x30, x9, [x10, #0x100]\n\t"
+                   "str w8, [x10, #0x120]\n\t"
                    "mrs x9, FPCR\n\t"
                    "str w9, [x10, #0x128]\n\t"
                    "mrs x9, FPSR\n\t"
@@ -1596,11 +1601,11 @@ __ASM_GLOBAL_FUNC( __wine_syscall_dispatcher,
                    __ASM_CFI(".cfi_offset 28, -0x68\n\t")
                    "and x20, x8, #0xfff\n\t"    /* syscall number */
                    "ubfx x21, x8, #12, #2\n\t"  /* syscall table number */
-                   "ldr x16, [x18, #0x2f8]\n\t" /* arm64_thread_data()->syscall_table */
+                   "ldr x16, [x18, #0x370]\n\t" /* thread_data->syscall_table */
                    "add x21, x16, x21, lsl #5\n\t"
                    "ldr x16, [x21, #16]\n\t"    /* table->ServiceLimit */
                    "cmp x20, x16\n\t"
-                   "bcs 4f\n\t"
+                   "bcs " __ASM_LOCAL_LABEL("bad_syscall") "\n\t"
                    "ldr x16, [x21, #24]\n\t"    /* table->ArgumentTable */
                    "ldrb w9, [x16, x20]\n\t"
                    "subs x9, x9, #64\n\t"
@@ -1613,8 +1618,10 @@ __ASM_GLOBAL_FUNC( __wine_syscall_dispatcher,
                    "str x10, [sp, x9]\n\t"
                    "cbnz x9, 1b\n"
                    "2:\tldr x16, [x21]\n\t"     /* table->ServiceTable */
-                   "ldr x16, [x16, x20, lsl 3]\n\t"
-                   "blr x16\n\t"
+                   "ldr x23, [x16, x20, lsl 3]\n\t"
+                   "ldr w11, [x18, #0x380]\n\t" /* thread_data->syscall_trace */
+                   "cbnz x11, " __ASM_LOCAL_LABEL("trace_syscall") "\n\t"
+                   "blr x23\n\t"
                    "mov sp, x22\n"
                    __ASM_CFI_CFA_IS_AT2(sp, 0x98, 0x02) /* frame->syscall_cfa */
                    __ASM_LOCAL_LABEL("__wine_syscall_dispatcher_return") ":\n\t"
@@ -1667,13 +1674,40 @@ __ASM_GLOBAL_FUNC( __wine_syscall_dispatcher,
                    /* switch to user stack */
                    "mov sp, x17\n\t"
                    "ret x16\n"
-                   "4:\tmov x0, #0xc0000000\n\t" /* STATUS_INVALID_SYSTEM_SERVICE */
+
+                   __ASM_LOCAL_LABEL("trace_syscall") ":\n\t"
+                   "stp x0, x1, [sp, #-0x40]!\n\t"
+                   "stp x2, x3, [sp, #0x10]\n\t"
+                   "stp x4, x5, [sp, #0x20]\n\t"
+                   "stp x6, x7, [sp, #0x30]\n\t"
+                   "mov x0, x8\n\t"             /* id */
+                   "mov x1, sp\n\t"             /* args */
+                   "ldr x16, [x21, #24]\n\t"    /* table->ArgumentTable */
+                   "ldrb w2, [x16, x20]\n\t"    /* len */
+                   "bl " __ASM_NAME("trace_syscall") "\n\t"
+                   "ldp x2, x3, [sp, #0x10]\n\t"
+                   "ldp x4, x5, [sp, #0x20]\n\t"
+                   "ldp x6, x7, [sp, #0x30]\n\t"
+                   "ldp x0, x1, [sp], #0x40\n\t"
+                   "blr x23\n"
+                   "mov sp, x22\n"
+
+                   __ASM_LOCAL_LABEL("trace_syscall_ret") ":\n\t"
+                   "mov x21, x0\n\t"            /* retval */
+                   "ldr w0, [sp, #0x120]\n\t"   /* frame->syscall_id */
+                   "mov x1, x21\n\t"            /* retval */
+                   "bl " __ASM_NAME("trace_sysret") "\n\t"
+                   "mov x0, x21\n\t"            /* retval */
+                   "b " __ASM_LOCAL_LABEL("__wine_syscall_dispatcher_return") "\n"
+
+                   __ASM_LOCAL_LABEL("bad_syscall") ":\n\t"
+                   "mov x0, #0xc0000000\n\t"    /* STATUS_INVALID_SYSTEM_SERVICE */
                    "movk x0, #0x001c\n\t"
                    "b " __ASM_LOCAL_LABEL("__wine_syscall_dispatcher_return") )
 
 __ASM_GLOBAL_FUNC( __wine_syscall_dispatcher_return,
-                   "mov sp, x0\n\t"
-                   "mov x0, x1\n\t"
+                   "ldr w11, [x18, #0x380]\n\t" /* thread_data->syscall_trace */
+                   "cbnz x11, " __ASM_LOCAL_LABEL("trace_syscall_ret") "\n\t"
                    "b " __ASM_LOCAL_LABEL("__wine_syscall_dispatcher_return") )
 
 
@@ -1681,7 +1715,7 @@ __ASM_GLOBAL_FUNC( __wine_syscall_dispatcher_return,
  *           __wine_unix_call_dispatcher
  */
 __ASM_GLOBAL_FUNC( __wine_unix_call_dispatcher,
-                   "ldr x10, [x18, #0x2f0]\n\t" /* arm64_thread_data()->syscall_frame */
+                   "ldr x10, [x18, #0x378]\n\t" /* thread_data->syscall_frame */
                    "stp x18, x19, [x10, #0x90]\n\t"
                    "stp x20, x21, [x10, #0xa0]\n\t"
                    "stp x22, x23, [x10, #0xb0]\n\t"
