@@ -2473,6 +2473,7 @@ end:
 
 static HRESULT PropertyStorage_WriteToStream(PropertyStorage_impl *This)
 {
+    const ULARGE_INTEGER size = { .QuadPart = 0 };
     PROPERTYSECTIONHEADER sectionHdr;
     HRESULT hr;
     ULONG count;
@@ -2480,6 +2481,10 @@ static HRESULT PropertyStorage_WriteToStream(PropertyStorage_impl *This)
     DWORD numProps, prop, sectionOffset, dwTemp;
     PROPVARIANT var;
 
+    hr = IStream_SetSize(This->stm, size);
+    if (FAILED(hr))
+        return hr;
+
     PropertyStorage_WriteHeadersToStream(This);
 
     /* Count properties.  Always at least one property, the code page */
@@ -2499,10 +2504,10 @@ static HRESULT PropertyStorage_WriteToStream(PropertyStorage_impl *This)
     seek.QuadPart = SECTIONHEADER_OFFSET;
     hr = IStream_Seek(This->stm, seek, STREAM_SEEK_SET, NULL);
     if (FAILED(hr))
-        goto end;
+        return hr;
     hr = IStream_Write(This->stm, &sectionHdr, sizeof(sectionHdr), &count);
     if (FAILED(hr))
-        goto end;
+        return hr;
 
     prop = 0;
     sectionOffset = sizeof(PROPERTYSECTIONHEADER) +
@@ -2513,7 +2518,7 @@ static HRESULT PropertyStorage_WriteToStream(PropertyStorage_impl *This)
         prop++;
         hr = PropertyStorage_WriteDictionaryToStream(This, &sectionOffset);
         if (FAILED(hr))
-            goto end;
+            return hr;
     }
 
     PropVariantInit(&var);
@@ -2523,7 +2528,7 @@ static HRESULT PropertyStorage_WriteToStream(PropertyStorage_impl *This)
     hr = PropertyStorage_WritePropertyToStream(This, prop++, PID_CODEPAGE,
      &var, &sectionOffset);
     if (FAILED(hr))
-        goto end;
+        return hr;
 
     if (This->locale != LOCALE_SYSTEM_DEFAULT)
     {
@@ -2532,7 +2537,7 @@ static HRESULT PropertyStorage_WriteToStream(PropertyStorage_impl *This)
         hr = PropertyStorage_WritePropertyToStream(This, prop++, PID_LOCALE,
          &var, &sectionOffset);
         if (FAILED(hr))
-            goto end;
+            return hr;
     }
 
     if (This->grfFlags & PROPSETFLAG_CASE_SENSITIVE)
@@ -2542,22 +2547,22 @@ static HRESULT PropertyStorage_WriteToStream(PropertyStorage_impl *This)
         hr = PropertyStorage_WritePropertyToStream(This, prop++, PID_BEHAVIOR,
          &var, &sectionOffset);
         if (FAILED(hr))
-            goto end;
+            return hr;
     }
 
     hr = PropertyStorage_WritePropertiesToStream(This, prop, &sectionOffset);
     if (FAILED(hr))
-        goto end;
+        return hr;
 
     /* Now write the byte count of the section */
     seek.QuadPart = SECTIONHEADER_OFFSET;
     hr = IStream_Seek(This->stm, seek, STREAM_SEEK_SET, NULL);
     if (FAILED(hr))
-        goto end;
+        return hr;
     StorageUtl_WriteDWord(&dwTemp, 0, sectionOffset);
     hr = IStream_Write(This->stm, &dwTemp, sizeof(dwTemp), &count);
-
-end:
+    if (SUCCEEDED(hr))
+        This->dirty = FALSE;
     return hr;
 }
 
@@ -2670,6 +2675,7 @@ static HRESULT PropertyStorage_ConstructEmpty(IStream *stm, REFFMTID rfmtid,
     hr = PropertyStorage_BaseConstruct(stm, rfmtid, grfMode, &ps);
     if (SUCCEEDED(hr))
     {
+        ps->dirty = TRUE;
         if (clsid)
             ps->clsid = *clsid;
         ps->format = 0;
