@@ -29,6 +29,7 @@
 #include "winbase.h"
 #include "winreg.h"
 #include "ntuser.h"
+#include "ntgdi.h"
 #include "malloc.h"
 
 #include "unixlib.h"
@@ -124,6 +125,7 @@ INT WINAPI wglChoosePixelFormat(HDC hdc, const PIXELFORMATDESCRIPTOR* ppfd)
     PIXELFORMATDESCRIPTOR format, best;
     int i, count, best_format;
     int bestDBuffer = -1, bestStereo = -1;
+    DWORD is_memdc;
 
     TRACE( "%p %p: size %u version %u flags %lu type %u color %u %u,%u,%u,%u "
            "accum %u depth %u stencil %u aux %u\n",
@@ -134,6 +136,8 @@ INT WINAPI wglChoosePixelFormat(HDC hdc, const PIXELFORMATDESCRIPTOR* ppfd)
     count = wglDescribePixelFormat( hdc, 0, 0, NULL );
     if (!count) return 0;
 
+    if (!NtGdiGetDCDword( hdc, NtGdiIsMemDC, &is_memdc )) is_memdc = 0;
+
     best_format = 0;
     best.dwFlags = 0;
     best.cAlphaBits = -1;
@@ -173,6 +177,11 @@ INT WINAPI wglChoosePixelFormat(HDC hdc, const PIXELFORMATDESCRIPTOR* ppfd)
             TRACE( "PFD_SUPPORT_OPENGL required but not found for iPixelFormat=%d\n", i );
             continue;
         }
+        if (is_memdc && ppfd->cColorBits == 16 && ppfd->cAlphaBits != 1)
+        {
+            TRACE( "Ignoring iPixelFormat=%d RGB565 format on memory DC\n", i );
+            continue;
+        }
 
         /* The behavior of PDF_STEREO/PFD_STEREO_DONTCARE and PFD_DOUBLEBUFFER / PFD_DOUBLEBUFFER_DONTCARE
          * is not very clear on MSDN. They specify that ChoosePixelFormat tries to match pixel formats
@@ -314,6 +323,7 @@ static struct wgl_pixel_format *get_pixel_formats( HDC hdc, UINT *num_formats,
     struct get_pixel_formats_params args = { .teb = NtCurrentTeb(), .hdc = hdc };
     PVOID *glReserved = NtCurrentTeb()->glReserved1;
     NTSTATUS status;
+    DWORD is_memdc;
 
     if (glReserved[WINE_GL_RESERVED_FORMATS_HDC] == hdc)
     {
@@ -329,6 +339,9 @@ static struct wgl_pixel_format *get_pixel_formats( HDC hdc, UINT *num_formats,
     args.max_formats = args.num_formats;
     if ((status = UNIX_CALL( get_pixel_formats, &args ))) goto error;
 
+    if (NtGdiGetDCDword( hdc, NtGdiIsMemDC, &is_memdc ) && is_memdc)
+        args.num_onscreen_formats = args.num_formats;
+
     *num_formats = args.num_formats;
     *num_onscreen_formats = args.num_onscreen_formats;
 
@@ -704,10 +717,13 @@ BOOL WINAPI wglChoosePixelFormatARB( HDC hdc, const int *attribs_int, const FLOA
     UINT i, num_wgl_formats, num_wgl_onscreen_formats;
     const struct wgl_pixel_format **format_array;
     struct compare_formats_ctx ctx = { 0 };
+    DWORD is_memdc;
 
     TRACE( "hdc %p, attribs_int %p, attribs_float %p, max_formats %u, formats %p, num_formats %p\n",
            hdc, attribs_int, attribs_float, max_formats, formats, num_formats );
 
+    if (!NtGdiGetDCDword( hdc, NtGdiIsMemDC, &is_memdc )) is_memdc = 0;
+
     wgl_formats = get_pixel_formats( hdc, &num_wgl_formats, &num_wgl_onscreen_formats );
 
     /* If the driver doesn't yet provide ARB attrib information in
@@ -747,7 +763,12 @@ BOOL WINAPI wglChoosePixelFormatARB( HDC hdc, const int *attribs_int, const FLOA
     /* Initialize the format_array with (pointers to) all wgl formats */
     format_array = malloc( num_wgl_formats * sizeof(*format_array) );
     if (!format_array) return FALSE;
-    for (i = 0; i < num_wgl_formats; ++i) format_array[i] = &wgl_formats[i];
+    for (i = 0; i < num_wgl_formats; ++i)
+    {
+        struct wgl_pixel_format *format = wgl_formats + i;
+        if (is_memdc && format->pfd.cColorBits == 16 && format->pfd.cAlphaBits != 1) format_array[i] = NULL;
+        else format_array[i] = &wgl_formats[i];
+    }
 
     /* Remove formats that are not acceptable */
     for (i = 0; i < ctx.num_attribs; ++i)
