@@ -180,13 +180,24 @@ static HRESULT PROPVAR_ConvertNumber(REFPROPVARIANT pv, int dest_bits,
 
 HRESULT WINAPI PropVariantToDouble(REFPROPVARIANT propvarIn, double *ret)
 {
+    HRESULT hr = S_OK;
     LONGLONG res;
-    HRESULT hr;
 
     TRACE("(%p, %p)\n", propvarIn, ret);
 
-    hr = PROPVAR_ConvertNumber(propvarIn, 64, TRUE, &res);
-    if (SUCCEEDED(hr)) *ret = (double)res;
+    switch (propvarIn->vt)
+    {
+        case VT_R8:
+            *ret = propvarIn->dblVal;
+            break;
+        case VT_R4:
+            *ret = propvarIn->fltVal;
+            break;
+        default:
+            hr = PROPVAR_ConvertNumber(propvarIn, 64, TRUE, &res);
+            if (SUCCEEDED(hr)) *ret = (double)res;
+    }
+
     return hr;
 }
 
@@ -706,6 +717,17 @@ HRESULT WINAPI PropVariantChangeType(PROPVARIANT *ppropvarDest, REFPROPVARIANT p
         }
         return hr;
     }
+    case VT_R8:
+    {
+        double res;
+        hr = PropVariantToDouble(propvarSrc, &res);
+        if (SUCCEEDED(hr))
+        {
+            ppropvarDest->vt = VT_R8;
+            ppropvarDest->dblVal = res;
+        }
+        return hr;
+    }
 
     case VT_LPWSTR:
     case VT_BSTR:
