@@ -49,11 +49,8 @@ static const struct vulkan_driver_funcs *driver_funcs;
 struct surface
 {
     struct vulkan_surface obj;
-    void *driver_private;
+    struct client_surface *client;
     HWND hwnd;
-
-    struct list entry;
-    struct rb_entry window_entry;
 };
 
 static struct surface *surface_from_handle( VkSurfaceKHR handle )
@@ -83,7 +80,6 @@ static VkResult win32u_vkCreateWin32SurfaceKHR( VkInstance client_instance, cons
     struct surface *surface;
     HWND dummy = NULL;
     VkResult res;
-    WND *win;
 
     TRACE( "client_instance %p, create_info %p, allocator %p, ret %p\n", client_instance, create_info, allocator, ret );
     if (allocator) FIXME( "Support for allocation callbacks not implemented yet\n" );
@@ -101,22 +97,13 @@ static VkResult win32u_vkCreateWin32SurfaceKHR( VkInstance client_instance, cons
         surface->hwnd = dummy;
     }
 
-    if ((res = driver_funcs->p_vulkan_surface_create( surface->hwnd, instance->host.instance,
-                                                      &host_surface, &surface->driver_private )))
+    if ((res = driver_funcs->p_vulkan_surface_create( surface->hwnd, instance, &host_surface, &surface->client )))
     {
         if (dummy) NtUserDestroyWindow( dummy );
         free( surface );
         return res;
     }
 
-    if (!(win = get_win_ptr( surface->hwnd )) || win == WND_DESKTOP || win == WND_OTHER_PROCESS)
-        list_init( &surface->entry );
-    else
-    {
-        list_add_tail( &win->vulkan_surfaces, &surface->entry );
-        release_win_ptr( win );
-    }
-
     vulkan_object_init( &surface->obj.obj, host_surface );
     surface->obj.instance = instance;
     instance->p_insert_object( instance, &surface->obj.obj );
@@ -132,21 +119,14 @@ static void win32u_vkDestroySurfaceKHR( VkInstance client_instance, VkSurfaceKHR
 {
     struct vulkan_instance *instance = vulkan_instance_from_handle( client_instance );
     struct surface *surface = surface_from_handle( client_surface );
-    WND *win;
 
     if (!surface) return;
 
     TRACE( "instance %p, handle 0x%s, allocator %p\n", instance, wine_dbgstr_longlong( client_surface ), allocator );
     if (allocator) FIXME( "Support for allocation callbacks not implemented yet\n" );
 
-    if ((win = get_win_ptr( surface->hwnd )) && win != WND_DESKTOP && win != WND_OTHER_PROCESS)
-    {
-        list_remove( &surface->entry );
-        release_win_ptr( win );
-    }
-
     instance->p_vkDestroySurfaceKHR( instance->host.instance, surface->obj.host.surface, NULL /* allocator */ );
-    driver_funcs->p_vulkan_surface_destroy( surface->hwnd, surface->driver_private );
+    client_surface_release( surface->client );
 
     instance->p_remove_object( instance, &surface->obj.obj );
 
@@ -311,9 +291,6 @@ static VkResult win32u_vkCreateSwapchainKHR( VkDevice client_device, const VkSwa
     if (surface) create_info_host.surface = surface->obj.host.surface;
     if (old_swapchain) create_info_host.oldSwapchain = old_swapchain->obj.host.swapchain;
 
-    /* update the host surface to commit any pending size change */
-    driver_funcs->p_vulkan_surface_update( surface->hwnd, surface->driver_private );
-
     /* Windows allows client rect to be empty, but host Vulkan often doesn't, adjust extents back to the host capabilities */
     res = instance->p_vkGetPhysicalDeviceSurfaceCapabilitiesKHR( physical_device->host.physical_device, surface->obj.host.surface, &capabilities );
     if (res) return res;
@@ -439,8 +416,7 @@ static VkResult win32u_vkQueuePresentKHR( VkQueue client_queue, const VkPresentI
         struct surface *surface = swapchain->surface;
         RECT client_rect;
 
-        if (surface->hwnd)
-            driver_funcs->p_vulkan_surface_presented( surface->hwnd, surface->driver_private, swapchain_res );
+        client_surface_present( surface->client, NULL );
 
         if (swapchain_res < VK_SUCCESS) continue;
         if (!NtUserGetClientRect( surface->hwnd, &client_rect, NtUserGetDpiForWindow( surface->hwnd ) ))
@@ -509,25 +485,37 @@ static struct vulkan_funcs vulkan_funcs =
     .p_get_host_surface_extension = win32u_get_host_surface_extension,
 };
 
-static VkResult nulldrv_vulkan_surface_create( HWND hwnd, VkInstance instance, VkSurfaceKHR *surface, void **private )
+static const struct client_surface_funcs nulldrv_vulkan_surface_funcs;
+
+static VkResult nulldrv_vulkan_surface_create( HWND hwnd, const struct vulkan_instance *instance, VkSurfaceKHR *surface,
+                                               struct client_surface **client )
 {
-    FIXME( "stub!\n" );
-    return VK_ERROR_INCOMPATIBLE_DRIVER;
+    VkHeadlessSurfaceCreateInfoEXT create_info = {.sType = VK_STRUCTURE_TYPE_HEADLESS_SURFACE_CREATE_INFO_EXT};
+    VkResult res;
+
+    if (!(*client = client_surface_create(sizeof(**client), &nulldrv_vulkan_surface_funcs, hwnd))) return VK_ERROR_OUT_OF_HOST_MEMORY;
+    if ((res = instance->p_vkCreateHeadlessSurfaceEXT( instance->host.instance, &create_info, NULL, surface )))
+    {
+        client_surface_release(*client);
+        *client = NULL;
+    }
+
+    return res;
 }
 
-static void nulldrv_vulkan_surface_destroy( HWND hwnd, void *private )
+static void nulldrv_vulkan_surface_destroy( struct client_surface *client )
 {
 }
 
-static void nulldrv_vulkan_surface_detach( HWND hwnd, void *private )
+static void nulldrv_vulkan_surface_detach( struct client_surface *client )
 {
 }
 
-static void nulldrv_vulkan_surface_update( HWND hwnd, void *private )
+static void nulldrv_vulkan_surface_update( struct client_surface *client )
 {
 }
 
-static void nulldrv_vulkan_surface_presented( HWND hwnd, void *private, VkResult result )
+static void nulldrv_vulkan_surface_present( struct client_surface *client, HDC hdc )
 {
 }
 
@@ -538,16 +526,20 @@ static VkBool32 nulldrv_vkGetPhysicalDeviceWin32PresentationSupportKHR( VkPhysic
 
 static const char *nulldrv_get_host_surface_extension(void)
 {
-    return "VK_WINE_nulldrv_surface";
+    return "VK_EXT_headless_surface";
 }
 
+static const struct client_surface_funcs nulldrv_vulkan_surface_funcs =
+{
+    .destroy = nulldrv_vulkan_surface_destroy,
+    .detach = nulldrv_vulkan_surface_detach,
+    .update = nulldrv_vulkan_surface_update,
+    .present = nulldrv_vulkan_surface_present,
+};
+
 static const struct vulkan_driver_funcs nulldrv_funcs =
 {
     .p_vulkan_surface_create = nulldrv_vulkan_surface_create,
-    .p_vulkan_surface_destroy = nulldrv_vulkan_surface_destroy,
-    .p_vulkan_surface_detach = nulldrv_vulkan_surface_detach,
-    .p_vulkan_surface_update = nulldrv_vulkan_surface_update,
-    .p_vulkan_surface_presented = nulldrv_vulkan_surface_presented,
     .p_vkGetPhysicalDeviceWin32PresentationSupportKHR = nulldrv_vkGetPhysicalDeviceWin32PresentationSupportKHR,
     .p_get_host_surface_extension = nulldrv_get_host_surface_extension,
 };
@@ -573,34 +565,11 @@ static void vulkan_driver_load(void)
     pthread_once( &init_once, vulkan_driver_init );
 }
 
-static VkResult lazydrv_vulkan_surface_create( HWND hwnd, VkInstance instance, VkSurfaceKHR *surface, void **private )
+static VkResult lazydrv_vulkan_surface_create( HWND hwnd, const struct vulkan_instance *instance, VkSurfaceKHR *surface,
+                                               struct client_surface **client )
 {
     vulkan_driver_load();
-    return driver_funcs->p_vulkan_surface_create( hwnd, instance, surface, private );
-}
-
-static void lazydrv_vulkan_surface_destroy( HWND hwnd, void *private )
-{
-    vulkan_driver_load();
-    return driver_funcs->p_vulkan_surface_destroy( hwnd, private );
-}
-
-static void lazydrv_vulkan_surface_detach( HWND hwnd, void *private )
-{
-    vulkan_driver_load();
-    return driver_funcs->p_vulkan_surface_detach( hwnd, private );
-}
-
-static void lazydrv_vulkan_surface_update( HWND hwnd, void *private )
-{
-    vulkan_driver_load();
-    return driver_funcs->p_vulkan_surface_update( hwnd, private );
-}
-
-static void lazydrv_vulkan_surface_presented( HWND hwnd, void *private, VkResult result )
-{
-    vulkan_driver_load();
-    driver_funcs->p_vulkan_surface_presented( hwnd, private, result );
+    return driver_funcs->p_vulkan_surface_create( hwnd, instance, surface, client );
 }
 
 static VkBool32 lazydrv_vkGetPhysicalDeviceWin32PresentationSupportKHR( VkPhysicalDevice device, uint32_t queue )
@@ -618,10 +587,6 @@ static const char *lazydrv_get_host_surface_extension(void)
 static const struct vulkan_driver_funcs lazydrv_funcs =
 {
     .p_vulkan_surface_create = lazydrv_vulkan_surface_create,
-    .p_vulkan_surface_destroy = lazydrv_vulkan_surface_destroy,
-    .p_vulkan_surface_detach = lazydrv_vulkan_surface_detach,
-    .p_vulkan_surface_update = lazydrv_vulkan_surface_update,
-    .p_vulkan_surface_presented = lazydrv_vulkan_surface_presented,
     .p_vkGetPhysicalDeviceWin32PresentationSupportKHR = lazydrv_vkGetPhysicalDeviceWin32PresentationSupportKHR,
     .p_get_host_surface_extension = lazydrv_get_host_surface_extension,
 };
@@ -652,25 +617,8 @@ static void vulkan_init_once(void)
     vulkan_funcs.p_vkGetDeviceProcAddr = p_vkGetDeviceProcAddr;
 }
 
-void vulkan_detach_surfaces( struct list *surfaces )
-{
-    struct surface *surface, *next;
-
-    LIST_FOR_EACH_ENTRY_SAFE( surface, next, surfaces, struct surface, entry )
-    {
-        driver_funcs->p_vulkan_surface_detach( surface->hwnd, surface->driver_private );
-        list_remove( &surface->entry );
-        list_init( &surface->entry );
-        surface->hwnd = NULL;
-    }
-}
-
 #else /* SONAME_LIBVULKAN */
 
-void vulkan_detach_surfaces( struct list *surfaces )
-{
-}
-
 static void vulkan_init_once(void)
 {
     ERR( "Wine was built without Vulkan support.\n" );
