@@ -1501,26 +1501,26 @@ static struct regsvr_decoder const decoder_list[] = {
 	gif_patterns
     },
     {   &CLSID_WICIcoDecoder,
-	"The Wine Project",
-	"ICO Decoder",
-	"1.0.0.0",
-	&GUID_VendorMicrosoft,
-	&GUID_ContainerFormatIco,
-	"image/vnd.microsoft.icon",
-	".ico",
-	ico_formats,
-	ico_patterns
+        "The Wine Project",
+        "ICO Decoder",
+        "1.0.0.0",
+        &GUID_VendorMicrosoft,
+        &GUID_ContainerFormatIco,
+        "image/ico,image/x-icon",
+        ".ico,.icon",
+        ico_formats,
+        ico_patterns
     },
     {   &CLSID_WICJpegDecoder,
-	"The Wine Project",
-	"JPEG Decoder",
-	"1.0.0.0",
-	&GUID_VendorMicrosoft,
-	&GUID_ContainerFormatJpeg,
-	"image/jpeg",
-	".jpg;.jpeg;.jfif",
-	jpeg_formats,
-	jpeg_patterns
+        "The Wine Project",
+        "JPEG Decoder",
+        "1.0.0.0",
+        &GUID_VendorMicrosoft,
+        &GUID_ContainerFormatJpeg,
+        "image/jpeg,image/jpe,image/jpg",
+        ".jpeg,.jpe,.jpg,.jfif,.exif",
+        jpeg_formats,
+        jpeg_patterns
     },
     {   &CLSID_WICWmpDecoder,
 	"The Wine Project",
@@ -1545,15 +1545,15 @@ static struct regsvr_decoder const decoder_list[] = {
 	png_patterns
     },
     {   &CLSID_WICTiffDecoder,
-	"The Wine Project",
-	"TIFF Decoder",
-	"1.0.0.0",
-	&GUID_VendorMicrosoft,
-	&GUID_ContainerFormatTiff,
-	"image/tiff",
-	".tif;.tiff",
-	tiff_decode_formats,
-	tiff_patterns
+        "The Wine Project",
+        "TIFF Decoder",
+        "1.0.0.0",
+        &GUID_VendorMicrosoft,
+        &GUID_ContainerFormatTiff,
+        "image/tiff,image/tif",
+        ".tiff,.tif",
+        tiff_decode_formats,
+        tiff_patterns
     },
     { NULL }			/* list terminator */
 };
@@ -1681,6 +1681,7 @@ static GUID const * const converter_formats[] = {
     &GUID_WICPixelFormat32bppPRGBA,
     &GUID_WICPixelFormat32bppGrayFloat,
     &GUID_WICPixelFormat48bppRGB,
+    &GUID_WICPixelFormat48bppRGBHalf,
     &GUID_WICPixelFormat64bppRGBA,
     &GUID_WICPixelFormat32bppCMYK,
     &GUID_WICPixelFormat96bppRGBFloat,
