@@ -45,6 +45,7 @@
 #include <winbase.h>
 #include <bthsdpdef.h>
 #include <bluetoothapis.h>
+#include <bthledef.h>
 #include <bthdef.h>
 #include <wine/winebth.h>
 
@@ -119,6 +120,7 @@ const int bluez_timeout = -1;
 #define BLUEZ_INTERFACE_DEVICE  "org.bluez.Device1"
 #define BLUEZ_INTERFACE_AGENT_MANAGER "org.bluez.AgentManager1"
 #define BLUEZ_INTERFACE_AGENT "org.bluez.Agent1"
+#define BLUEZ_INTERFACE_GATT_SERVICE "org.bluez.GattService1"
 
 #define DO_FUNC( f ) typeof( f ) (*p_##f)
 DBUS_FUNCS;
@@ -306,6 +308,19 @@ static void parse_mac_address( const char *addr_str, BYTE dest[6] )
         dest[i] = addr[i];
 }
 
+static BOOL parse_uuid( GUID *guid, const char *str )
+{
+    int ret;
+    if (strlen( str ) != 36)
+        return FALSE;
+    ret = sscanf( str, "%08x-%hx-%hx-%2hhx%2hhx-%2hhx%2hhx%2hhx%2hhx%2hhx%2hhx", &guid->Data1, &guid->Data2, &guid->Data3,
+                  &guid->Data4[0], &guid->Data4[1], &guid->Data4[2], &guid->Data4[3], &guid->Data4[4], &guid->Data4[5],
+                  &guid->Data4[6], &guid->Data4[7] );
+    if (ret != 11)
+        return FALSE;
+    return TRUE;
+}
+
 static void bluez_dbus_wait_for_reply_callback( DBusPendingCall *pending_call, void *wait )
 {
     sem_post( wait );
@@ -840,6 +855,8 @@ struct bluez_watcher_ctx
     struct list initial_radio_list;
     /* struct bluez_init_entry */
     struct list initial_device_list;
+    /* struct bluez_init_entry */
+    struct list initial_gatt_service_list;
 
     /* struct bluez_watcher_event */
     struct list event_list;
@@ -850,6 +867,7 @@ struct bluez_init_entry
     union {
         struct winebluetooth_watcher_event_radio_added radio;
         struct winebluetooth_watcher_event_device_added device;
+        struct winebluetooth_watcher_event_gatt_service_added service;
     } object;
     struct list entry;
 };
@@ -1266,6 +1284,75 @@ NTSTATUS bluez_device_disconnect( void *connection, const char *device_path )
     return STATUS_SUCCESS;
 }
 
+static BOOL bluez_event_list_queue_new_event( struct list *event_list,
+                                              enum winebluetooth_watcher_event_type event_type,
+                                              union winebluetooth_watcher_event_data event );
+struct bluez_device_pair_data
+{
+    IRP *irp;
+    struct bluez_watcher_ctx *watcher_ctx;
+};
+
+static void bluez_device_pair_callback( DBusPendingCall *pending, void *param )
+{
+    struct bluez_device_pair_data *data = param;
+    DBusMessage *reply;
+    DBusError error;
+    union winebluetooth_watcher_event_data event = {0};
+
+    event.pairing_finished.irp = data->irp;
+    reply = p_dbus_pending_call_steal_reply( pending );
+    p_dbus_error_init( &error );
+    if (p_dbus_set_error_from_message( &error, reply ))
+    {
+        event.pairing_finished.result = bluez_dbus_error_to_ntstatus( &error );
+        ERR( "Failed to pair: %s: %s\n", debugstr_a( error.name ), debugstr_a( error.message ) );
+    }
+    p_dbus_error_free( &error );
+
+    bluez_event_list_queue_new_event( &data->watcher_ctx->event_list,
+                                      BLUETOOTH_WATCHER_EVENT_TYPE_PAIRING_FINISHED, event );
+    p_dbus_message_unref( reply );
+}
+
+NTSTATUS bluez_device_start_pairing( void *connection, void *watcher_ctx, struct unix_name *device, IRP *irp )
+{
+    DBusMessage *request;
+    DBusPendingCall *pending_call = NULL;
+    struct bluez_device_pair_data *data;
+    dbus_bool_t success;
+
+    TRACE( "(%p, %p, %s, %p)\n", connection, watcher_ctx, debugstr_a( device->str ), irp );
+
+    request = p_dbus_message_new_method_call( BLUEZ_DEST, device->str, BLUEZ_INTERFACE_DEVICE, "Pair" );
+    if (!request)
+        return STATUS_NO_MEMORY;
+
+    data = malloc( sizeof( *data ) );
+    if (!data)
+    {
+        p_dbus_message_unref( request );
+        return STATUS_NO_MEMORY;
+    }
+    data->irp = irp;
+    data->watcher_ctx = watcher_ctx;
+    success = p_dbus_connection_send_with_reply( connection, request, &pending_call, bluez_timeout );
+    p_dbus_message_unref( request );
+    if (!success)
+        return STATUS_NO_MEMORY;
+    if (!pending_call)
+        return STATUS_INTERNAL_ERROR;
+    if (!p_dbus_pending_call_set_notify( pending_call, bluez_device_pair_callback, data, free ))
+    {
+        p_dbus_pending_call_cancel( pending_call );
+        p_dbus_pending_call_unref( pending_call );
+        return STATUS_NO_MEMORY;
+    }
+
+    p_dbus_pending_call_unref( pending_call );
+    return STATUS_PENDING;
+}
+
 struct bluez_watcher_event
 {
     struct list entry;
@@ -1659,7 +1746,7 @@ static DBusHandlerResult bluez_filter( DBusConnection *conn, DBusMessage *msg, v
                 if (status != STATUS_SUCCESS)
                 {
                     ERR( "Failed to create async call to get adapter properties: %#x\n",
-                         (int)status );
+                         status );
                     unix_name_free( radio );
                     return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
                 }
@@ -1741,7 +1828,7 @@ static DBusHandlerResult bluez_filter( DBusConnection *conn, DBusMessage *msg, v
                 status = bluez_device_get_props_by_path_async( conn, device->str, &pending_call );
                 if (status)
                 {
-                    ERR( "Failed to create async call to get device properties: %#x\n", (int)status );
+                    ERR( "Failed to create async call to get device properties: %#x\n", status );
                     unix_name_free( device );
                     return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
                 }
@@ -1825,6 +1912,12 @@ static void bluez_watcher_free( struct bluez_watcher_ctx *watcher )
         case BLUETOOTH_WATCHER_EVENT_TYPE_DEVICE_PROPERTIES_CHANGED:
             unix_name_free( (struct unix_name *)event1->event.device_props_changed.device.handle );
             break;
+        case BLUETOOTH_WATCHER_EVENT_TYPE_PAIRING_FINISHED:
+            break;
+        case BLUETOOTH_WATCHER_EVENT_TYPE_DEVICE_GATT_SERVICE_ADDED:
+            unix_name_free( (struct unix_name *)event1->event.gatt_service_added.device.handle );
+            unix_name_free( (struct unix_name *)event1->event.gatt_service_added.service.handle );
+            break;
         }
         free( event1 );
     }
@@ -1846,12 +1939,13 @@ NTSTATUS bluez_watcher_init( void *connection, void **ctx )
     if (status != STATUS_SUCCESS)
     {
         free( watcher_ctx );
-        ERR( "could not create async GetManagedObjects call: %#x\n", (int)status);
+        ERR( "could not create async GetManagedObjects call: %#x\n", status);
         return status;
     }
     watcher_ctx->init_device_list_call = call;
     list_init( &watcher_ctx->initial_radio_list );
     list_init( &watcher_ctx->initial_device_list );
+    list_init( &watcher_ctx->initial_gatt_service_list );
     list_init( &watcher_ctx->event_list );
 
     /* The bluez_dbus_loop thread will free up the watcher when the disconnect message is processed (i.e,
@@ -1907,7 +2001,7 @@ void bluez_watcher_close( void *connection, void *ctx )
 }
 
 static NTSTATUS bluez_build_initial_device_lists( DBusMessage *reply, struct list *adapter_list,
-                                                  struct list *device_list )
+                                                  struct list *device_list, struct list *gatt_service_list )
 {
     DBusMessageIter dict, paths_iter, iface_iter, prop_iter;
     const char *path;
@@ -2015,10 +2109,82 @@ static NTSTATUS bluez_build_initial_device_lists( DBusMessage *reply, struct lis
                 TRACE( "Found BlueZ org.bluez.Device1 object %s: %p\n", debugstr_a( path ), device_name );
                 break;
             }
+            else if (!strcmp( iface, BLUEZ_INTERFACE_GATT_SERVICE ))
+            {
+                struct unix_name *service_name, *device_name = NULL;
+                struct bluez_init_entry *init_device;
+                DBusMessageIter variant;
+                const char *prop_name;
+
+                init_device = calloc( 1, sizeof( *init_device ) );
+                if (!init_device)
+                {
+                    status = STATUS_NO_MEMORY;
+                    goto done;
+                }
+
+                service_name = unix_name_get_or_create( path );
+                if (!service_name)
+                {
+                    ERR( "Failed to allocate memory for service path %s\n", debugstr_a( path ) );
+                    break;
+                }
+                init_device->object.service.service.handle = (UINT_PTR)service_name;
+
+                while ((prop_name = bluez_next_dict_entry( &prop_iter, &variant )))
+                {
+                    if (!strcmp( prop_name, "Device" )
+                        && p_dbus_message_iter_get_arg_type( &variant ) == DBUS_TYPE_OBJECT_PATH )
+                    {
+                        const char *device_path;
+
+                        p_dbus_message_iter_get_basic( &variant, &device_path );
+                        device_name = unix_name_get_or_create( device_path );
+                        if (!device_name)
+                        {
+                            unix_name_free( service_name );
+                            free( init_device );
+                            ERR( "Failed to allocate memory for device path %s\n", debugstr_a( device_path ));
+                            status = STATUS_NO_MEMORY;
+                            goto done;
+                        }
+                        init_device->object.service.device.handle = (UINT_PTR)device_name;
+                    }
+                    else if (!strcmp( prop_name, "Handle" )
+                             && p_dbus_message_iter_get_arg_type( &variant ) == DBUS_TYPE_UINT16)
+                        p_dbus_message_iter_get_basic( &variant, &init_device->object.service.attr_handle );
+                    else if (!strcmp( prop_name, "Primary" )
+                             && p_dbus_message_iter_get_arg_type( &variant ) == DBUS_TYPE_BOOLEAN)
+                    {
+                        dbus_bool_t primary;
+                        p_dbus_message_iter_get_basic( &variant, &primary );
+                        init_device->object.service.is_primary = !!primary;
+                    }
+                    else if (!strcmp( prop_name, "UUID" )
+                             && p_dbus_message_iter_get_arg_type( &variant ) == DBUS_TYPE_STRING)
+                    {
+                        const char *uuid_str;
+                        p_dbus_message_iter_get_basic( &variant, &uuid_str );
+                        if (!parse_uuid( &init_device->object.service.uuid, uuid_str ))
+                            ERR("Failed to parse UUID %s for GATT service %s\n", debugstr_a( uuid_str ), path );
+                    }
+                }
+                if (!device_name)
+                {
+                    unix_name_free( service_name );
+                    free( init_device );
+                    ERR( "Could not find the associated device for the GATT service %s\n", debugstr_a( path ) );
+                    break;
+                }
+                list_add_tail( gatt_service_list, &init_device->entry );
+                TRACE( "Found BlueZ org.bluez.GattService1 object %s %p\n", debugstr_a( path ), service_name );
+                break;
+            }
         }
     }
 
-    TRACE( "Initial device list: radios: %d, devices: %d\n", list_count( adapter_list ), list_count( device_list ) );
+    TRACE( "Initial device list: radios: %d, devices: %d, GATT services: %d\n", list_count( adapter_list ),
+           list_count( device_list ), list_count( gatt_service_list ) );
  done:
     return status;
 }
@@ -2047,6 +2213,17 @@ static BOOL bluez_watcher_event_queue_ready( struct bluez_watcher_ctx *ctx, stru
         free( device );
         return TRUE;
     }
+    if (!list_empty( &ctx->initial_gatt_service_list ))
+    {
+        struct bluez_init_entry *service;
+
+        service = LIST_ENTRY( list_head( &ctx->initial_gatt_service_list ), struct bluez_init_entry, entry );
+        event->event_type = BLUETOOTH_WATCHER_EVENT_TYPE_DEVICE_GATT_SERVICE_ADDED;
+        event->event_data.gatt_service_added = service->object.service;
+        list_remove( &service->entry );
+        free( service );
+        return TRUE;
+    }
     if (!list_empty( &ctx->event_list ))
     {
         struct bluez_watcher_event *watcher_event =
@@ -2143,11 +2320,12 @@ NTSTATUS bluez_dbus_loop( void *c, void *watcher, void *auth_agent,
                 return STATUS_NO_MEMORY;
             }
             status = bluez_build_initial_device_lists( reply, &watcher_ctx->initial_radio_list,
-                                                       &watcher_ctx->initial_device_list );
+                                                       &watcher_ctx->initial_device_list,
+                                                       &watcher_ctx->initial_gatt_service_list );
             p_dbus_message_unref( reply );
             if (status != STATUS_SUCCESS)
             {
-                WARN( "Error building initial bluetooth devices list: %#x\n", (int)status );
+                WARN( "Error building initial bluetooth devices list: %#x\n", status );
                 p_dbus_connection_unref( connection );
                 bluez_auth_agent_ctx_decref( auth_agent );
                 return status;
@@ -2195,5 +2373,9 @@ NTSTATUS bluez_device_disconnect( void *connection, const char *device_path )
 {
     return STATUS_NOT_SUPPORTED;
 }
+NTSTATUS bluez_device_start_pairing( void *connection, void *watcher_ctx, struct unix_name *device, IRP *irp )
+{
+    return STATUS_NOT_SUPPORTED;
+}
 
 #endif /* SONAME_LIBDBUS_1 */
