@@ -54,6 +54,11 @@ struct bluetooth_device_free_params
     unix_name_t device;
 };
 
+struct bluetooth_gatt_service_free_params
+{
+    unix_name_t service;
+};
+
 struct bluetooth_device_disconnect_params
 {
     unix_name_t device;
@@ -100,6 +105,12 @@ struct bluetooth_auth_send_response_params
     BOOL *authenticated;
 };
 
+struct bluetooth_device_start_pairing_params
+{
+    unix_name_t device;
+    IRP *irp;
+};
+
 struct bluetooth_get_event_params
 {
     struct winebluetooth_event result;
@@ -119,10 +130,13 @@ enum bluetoothapis_funcs
 
     unix_bluetooth_device_free,
     unix_bluetooth_device_disconnect,
+    unix_bluetooth_device_start_pairing,
 
     unix_bluetooth_auth_agent_enable_incoming,
     unix_bluetooth_auth_send_response,
 
+    unix_bluetooth_gatt_service_free,
+
     unix_bluetooth_get_event,
 
     unix_funcs_count
