@@ -53,6 +53,7 @@ static const struct subtype_info subtype_info_list[] =
     { &MFVideoFormat_UYVY,   16, MAKEFOURCC('U', 'Y', 'V', 'Y') },
     { &MFVideoFormat_YVYU,   16, MAKEFOURCC('Y', 'V', 'Y', 'U') },
     { &MFVideoFormat_NV11,   12, MAKEFOURCC('N', 'V', '1', '1') },
+    { &MFVideoFormat_P010,   24, MAKEFOURCC('P', '0', '1', '0') },
     { &MFVideoFormat_RGB8,   8,  BI_RGB },
     { &MFVideoFormat_RGB555, 16, BI_RGB },
     { &MFVideoFormat_RGB565, 16, BI_BITFIELDS },
@@ -175,10 +176,14 @@ static ULONG WINAPI unknown_Release(IUnknown *iface)
             IMFMediaType_Release(decoder->input_type);
         if (decoder->output_type)
             IMFMediaType_Release(decoder->output_type);
+        if (decoder->stream_type)
+            IMFMediaType_Release(decoder->stream_type);
         if (decoder->output_attributes)
             IMFAttributes_Release(decoder->output_attributes);
         if (decoder->attributes)
             IMFAttributes_Release(decoder->attributes);
+        FreeMediaType(&decoder->dmo_input_type);
+        FreeMediaType(&decoder->dmo_output_type);
         wg_sample_queue_destroy(decoder->wg_sample_queue);
         free(decoder);
     }
@@ -944,6 +949,7 @@ static HRESULT WINAPI transform_ProcessOutput(IMFTransform *iface, DWORD flags,
     LONGLONG duration;
     IMFSample *sample;
     UINT64 frame_size, frame_rate;
+    bool preserve_timestamps;
     GUID subtype;
     DWORD size;
     HRESULT hr;
@@ -989,19 +995,27 @@ static HRESULT WINAPI transform_ProcessOutput(IMFTransform *iface, DWORD flags,
     }
 
     if (SUCCEEDED(hr = wg_transform_read_mf(decoder->wg_transform, sample,
-            sample_size, &samples->dwStatus)))
+            sample_size, &samples->dwStatus, &preserve_timestamps)))
     {
         wg_sample_queue_flush(decoder->wg_sample_queue, false);
 
-        if (FAILED(IMFMediaType_GetUINT64(decoder->input_type, &MF_MT_FRAME_RATE, &frame_rate)))
-            frame_rate = (UINT64)30000 << 32 | 1001;
+        if (decoder->IMediaObject_iface.lpVtbl)
+            duration = 0; /* WMV decoder doesn't output any timestamp or duration */
+        else
+        {
+            if (FAILED(IMFMediaType_GetUINT64(decoder->input_type, &MF_MT_FRAME_RATE, &frame_rate)))
+                frame_rate = (UINT64)30000 << 32 | 1001;
+            duration = MulDiv(10000000, (UINT32)frame_rate, frame_rate >> 32);
+        }
 
-        duration = (UINT64)10000000 * (UINT32)frame_rate / (frame_rate >> 32);
-        if (FAILED(IMFSample_SetSampleTime(sample, decoder->sample_time)))
-            WARN("Failed to set sample time\n");
-        if (FAILED(IMFSample_SetSampleDuration(sample, duration)))
-            WARN("Failed to set sample duration\n");
-        decoder->sample_time += duration;
+        if (!preserve_timestamps)
+        {
+            if (FAILED(IMFSample_SetSampleTime(sample, decoder->sample_time)))
+                WARN("Failed to set sample time\n");
+            if (FAILED(IMFSample_SetSampleDuration(sample, duration)))
+                WARN("Failed to set sample duration\n");
+            decoder->sample_time += duration;
+        }
     }
 
     if (hr == MF_E_TRANSFORM_STREAM_CHANGE)
@@ -1633,6 +1647,7 @@ static HRESULT video_decoder_create_with_types(const GUID *const *input_types, U
         goto failed;
 
     decoder->wg_transform_attrs.input_queue_length = 15;
+    decoder->wg_transform_attrs.preserve_timestamps = TRUE;
 
     *out = decoder;
     TRACE("Created decoder %p\n", decoder);
