@@ -64,7 +64,7 @@ static BOOL CALLBACK get_process_windows(HWND hwnd, LPARAM lp)
     return TRUE;
 }
 
-#include "pshpack1.h"
+#pragma pack(push,1)
 
 typedef struct
 {
@@ -86,7 +86,7 @@ typedef struct
     GRPICONDIRENTRY idEntries[1];
 } GRPICONDIR;
 
-#include "poppack.h"
+#pragma pack(pop)
 
 static void quit_reply(int reply)
 {
@@ -399,9 +399,6 @@ static BOOL process_attach(void)
     params.strings = strings;
     params.app_icon_callback = (UINT_PTR)macdrv_app_icon;
     params.app_quit_request_callback = (UINT_PTR)macdrv_app_quit_request;
-    params.dnd_query_drag_callback = (UINT_PTR)macdrv_dnd_query_drag;
-    params.dnd_query_drop_callback = (UINT_PTR)macdrv_dnd_query_drop;
-    params.dnd_query_exited_callback = (UINT_PTR)macdrv_dnd_query_exited;
 
     if (MACDRV_CALL(init, &params)) return FALSE;
 
