@@ -209,10 +209,10 @@ static BOOL wait_pa_operation_complete(pa_operation *o)
     if (!o)
         return FALSE;
 
-    while (pa_operation_get_state(o) == PA_OPERATION_RUNNING)
+    while (pulse_ml && pa_operation_get_state(o) == PA_OPERATION_RUNNING)
         pulse_cond_wait();
     pa_operation_unref(o);
-    return TRUE;
+    return !!pulse_ml;
 }
 
 /* Following pulseaudio design here, mainloop has the lock taken whenever
@@ -244,6 +244,7 @@ static NTSTATUS pulse_process_attach(void *args)
 
     pthread_mutexattr_init(&attr);
     pthread_mutexattr_setprotocol(&attr, PTHREAD_PRIO_INHERIT);
+    pthread_mutexattr_setrobust(&attr, PTHREAD_MUTEX_ROBUST);
 
     if (pthread_mutex_init(&pulse_mutex, &attr) != 0)
         pthread_mutex_init(&pulse_mutex, NULL);
@@ -275,6 +276,14 @@ static NTSTATUS pulse_process_detach(void *args)
     return STATUS_SUCCESS;
 }
 
+static void pulse_main_loop_thread_cleanup(void *context)
+{
+    TRACE("Main loop thread is being aborted.\n");
+
+    pulse_ml = NULL;
+    pulse_broadcast();
+}
+
 static NTSTATUS pulse_main_loop(void *args)
 {
     struct main_loop_params *params = args;
@@ -283,7 +292,9 @@ static NTSTATUS pulse_main_loop(void *args)
     pulse_ml = pa_mainloop_new();
     pa_mainloop_set_poll_func(pulse_ml, pulse_poll_func, NULL);
     NtSetEvent(params->event, NULL);
+    pthread_cleanup_push(pulse_main_loop_thread_cleanup, NULL);
     pa_mainloop_run(pulse_ml, &ret);
+    pthread_cleanup_pop(0);
     pa_mainloop_free(pulse_ml);
     pulse_unlock();
     return STATUS_SUCCESS;
@@ -1247,7 +1258,7 @@ static NTSTATUS pulse_release_stream(void *args)
     pulse_lock();
     if (PA_STREAM_IS_GOOD(pa_stream_get_state(stream->stream))) {
         pa_stream_disconnect(stream->stream);
-        while (PA_STREAM_IS_GOOD(pa_stream_get_state(stream->stream)))
+        while (pulse_ml && PA_STREAM_IS_GOOD(pa_stream_get_state(stream->stream)))
             pulse_cond_wait();
     }
     pa_stream_unref(stream->stream);
@@ -2494,8 +2505,6 @@ static NTSTATUS pulse_set_sample_rate(void *args)
     struct pulse_stream *stream = handle_get_stream(params->stream);
     HRESULT hr = S_OK;
     int success;
-    SIZE_T size, new_bufsize_frames;
-    BYTE *new_buffer = NULL;
     pa_sample_spec new_ss;
 
     pulse_lock();
@@ -2510,22 +2519,12 @@ static NTSTATUS pulse_set_sample_rate(void *args)
 
     new_ss = stream->ss;
     new_ss.rate = params->rate;
-    new_bufsize_frames = ceil((stream->duration / 10000000.) * new_ss.rate);
-    size = new_bufsize_frames * 2 * pa_frame_size(&stream->ss);
-
-    if (NtAllocateVirtualMemory(GetCurrentProcess(), (void **)&new_buffer,
-                                zero_bits, &size, MEM_COMMIT, PAGE_READWRITE)) {
-        hr = E_OUTOFMEMORY;
-        goto exit;
-    }
 
     if (!wait_pa_operation_complete(pa_stream_update_sample_rate(stream->stream, params->rate, pulse_op_cb, &success)))
         success = 0;
 
     if (!success) {
         hr = E_OUTOFMEMORY;
-        size = 0;
-        NtFreeVirtualMemory(GetCurrentProcess(), (void **)&new_buffer, &size, MEM_RELEASE);
         goto exit;
     }
 
@@ -2536,15 +2535,9 @@ static NTSTATUS pulse_set_sample_rate(void *args)
     stream->pa_offs_bytes = stream->lcl_offs_bytes = 0;
     stream->held_bytes = stream->pa_held_bytes = 0;
     stream->period_bytes = pa_frame_size(&new_ss) * muldiv(stream->mmdev_period_usec, new_ss.rate, 1000000);
-    stream->real_bufsize_bytes = size;
-    stream->bufsize_frames = new_bufsize_frames;
     stream->ss = new_ss;
 
-    size = 0;
-    NtFreeVirtualMemory(GetCurrentProcess(), (void **)&stream->local_buffer, &size, MEM_RELEASE);
-
-    silence_buffer(new_ss.format, new_buffer, size);
-    stream->local_buffer = new_buffer;
+    silence_buffer(new_ss.format, stream->local_buffer, stream->real_bufsize_bytes);
 
 exit:
     pulse_unlock();
@@ -2650,6 +2643,14 @@ fail:
     return STATUS_SUCCESS;
 }
 
+static NTSTATUS pulse_midi_get_driver(void *args)
+{
+    static const WCHAR driver[] = {'a','l','s','a',0};
+
+    memcpy( args, driver, sizeof(driver) );
+    return STATUS_SUCCESS;
+}
+
 const unixlib_entry_t __wine_unix_call_funcs[] =
 {
     pulse_process_attach,
@@ -2682,6 +2683,7 @@ const unixlib_entry_t __wine_unix_call_funcs[] =
     pulse_test_connect,
     pulse_is_started,
     pulse_get_prop_value,
+    pulse_midi_get_driver,
     pulse_not_implemented,
     pulse_not_implemented,
     pulse_not_implemented,
@@ -3180,6 +3182,7 @@ const unixlib_entry_t __wine_unix_call_wow64_funcs[] =
     pulse_wow64_test_connect,
     pulse_is_started,
     pulse_wow64_get_prop_value,
+    pulse_midi_get_driver,
     pulse_not_implemented,
     pulse_not_implemented,
     pulse_not_implemented,
