@@ -195,81 +195,6 @@ static DWORD set_reg_value_dword( HKEY hkey, const WCHAR *name, DWORD value )
 
 #if defined(__i386__) || defined(__x86_64__)
 
-extern UINT64 WINAPI do_xgetbv( unsigned int cx);
-#ifdef __i386__
-__ASM_STDCALL_FUNC( do_xgetbv, 4,
-                   "movl 4(%esp),%ecx\n\t"
-                   "xgetbv\n\t"
-                   "ret $4" )
-#else
-__ASM_GLOBAL_FUNC( do_xgetbv,
-                   "xgetbv\n\t"
-                   "shlq $32,%rdx\n\t"
-                   "orq %rdx,%rax\n\t"
-                   "ret" )
-#endif
-
-static void initialize_xstate_features(struct _KUSER_SHARED_DATA *data)
-{
-    static const ULONG64 wine_xstate_supported_features = 0xfc; /* XSTATE_AVX, XSTATE_MPX_BNDREGS, XSTATE_MPX_BNDCSR,
-                                                                 * XSTATE_AVX512_KMASK, XSTATE_AVX512_ZMM_H, XSTATE_AVX512_ZMM */
-    XSTATE_CONFIGURATION *xstate = &data->XState;
-    ULONG64 supported_mask;
-    unsigned int i, off;
-    int regs[4];
-
-    if (!data->ProcessorFeatures[PF_AVX_INSTRUCTIONS_AVAILABLE])
-        return;
-
-    __cpuidex(regs, 0, 0);
-
-    TRACE("Max cpuid level %#x.\n", regs[0]);
-    if (regs[0] < 0xd)
-        return;
-
-    __cpuidex(regs, 1, 0);
-    TRACE("CPU features %#x, %#x, %#x, %#x.\n", regs[0], regs[1], regs[2], regs[3]);
-    if (!(regs[2] & (0x1 << 27))) /* xsave OS enabled */
-        return;
-
-    __cpuidex(regs, 0xd, 0);
-    TRACE("XSAVE details %#x, %#x, %#x, %#x.\n", regs[0], regs[1], regs[2], regs[3]);
-    supported_mask = ((ULONG64)regs[3] << 32) | regs[0];
-    supported_mask &= do_xgetbv(0) & wine_xstate_supported_features;
-    if (!(supported_mask >> 2))
-        return;
-
-    xstate->EnabledFeatures = (1 << XSTATE_LEGACY_FLOATING_POINT) | (1 << XSTATE_LEGACY_SSE) | supported_mask;
-    xstate->EnabledVolatileFeatures = xstate->EnabledFeatures;
-    xstate->AllFeatureSize = regs[1];
-
-    __cpuidex(regs, 0xd, 1);
-    xstate->OptimizedSave = regs[0] & 1;
-    xstate->CompactionEnabled = !!(regs[0] & 2);
-
-    xstate->Features[0].Size = xstate->AllFeatures[0] = offsetof(XSAVE_FORMAT, XmmRegisters);
-    xstate->Features[1].Size = xstate->AllFeatures[1] = sizeof(M128A) * 16;
-    xstate->Features[1].Offset = xstate->Features[0].Size;
-    off = sizeof(XSAVE_FORMAT) + sizeof(XSAVE_AREA_HEADER);
-    supported_mask >>= 2;
-    for (i = 2; supported_mask; ++i, supported_mask >>= 1)
-    {
-        if (!(supported_mask & 1)) continue;
-        __cpuidex( regs, 0xd, i );
-        xstate->Features[i].Offset = regs[1];
-        xstate->Features[i].Size = xstate->AllFeatures[i] = regs[0];
-        if (regs[2] & 2)
-        {
-            xstate->AlignedFeatures |= (ULONG64)1 << i;
-            off = (off + 63) & ~63;
-        }
-        off += xstate->Features[i].Size;
-        TRACE("xstate[%d] offset %lu, size %lu, aligned %d.\n", i, xstate->Features[i].Offset, xstate->Features[i].Size, !!(regs[2] & 2));
-    }
-    xstate->Size = xstate->CompactionEnabled ? off : xstate->Features[i - 1].Offset + xstate->Features[i - 1].Size;
-    TRACE("xstate size %lu, compacted %d, optimized %d.\n", xstate->Size, xstate->CompactionEnabled, xstate->OptimizedSave);
-}
-
 static BOOL is_tsc_trusted_by_the_kernel(void)
 {
     char buf[4] = {0};
@@ -372,12 +297,17 @@ static UINT64 read_tsc_frequency(void)
     return freq;
 }
 
-#else
+#elif defined(__aarch64__)
 
-static void initialize_xstate_features(struct _KUSER_SHARED_DATA *data)
+static UINT64 read_tsc_frequency(void)
 {
+    UINT64 tsc_frequency;
+    __asm__ volatile( "mrs %[Res], CNTFRQ_EL0" : [Res] "=r" (tsc_frequency) );
+    return tsc_frequency;
 }
 
+#else
+
 static UINT64 read_tsc_frequency(void)
 {
     return 0;
@@ -389,14 +319,10 @@ static void create_user_shared_data(void)
 {
     struct _KUSER_SHARED_DATA *data;
     RTL_OSVERSIONINFOEXW version;
-    SYSTEM_CPU_INFORMATION sci;
-    SYSTEM_BASIC_INFORMATION sbi;
-    BOOLEAN *features;
     OBJECT_ATTRIBUTES attr = {sizeof(attr)};
     UNICODE_STRING name = RTL_CONSTANT_STRING( L"\\KernelObjects\\__wine_user_shared_data" );
     NTSTATUS status;
     HANDLE handle;
-    ULONG i;
 
     InitializeObjectAttributes( &attr, &name, OBJ_OPENIF, NULL, NULL );
     if ((status = NtOpenSection( &handle, SECTION_ALL_ACCESS, &attr )))
@@ -414,113 +340,19 @@ static void create_user_shared_data(void)
 
     version.dwOSVersionInfoSize = sizeof(version);
     RtlGetVersion( &version );
-    NtQuerySystemInformation( SystemBasicInformation, &sbi, sizeof(sbi), NULL );
-    NtQuerySystemInformation( SystemCpuInformation, &sci, sizeof(sci), NULL );
 
-    data->TickCountMultiplier         = 1 << 24;
-    data->LargePageMinimum            = 2 * 1024 * 1024;
     data->NtBuildNumber               = version.dwBuildNumber;
     data->NtProductType               = version.wProductType;
     data->ProductTypeIsValid          = TRUE;
-    data->NativeProcessorArchitecture = sci.ProcessorArchitecture;
     data->NtMajorVersion              = version.dwMajorVersion;
     data->NtMinorVersion              = version.dwMinorVersion;
     data->SuiteMask                   = version.wSuiteMask;
-    data->NumberOfPhysicalPages       = sbi.MmNumberOfPhysicalPages;
-    data->NXSupportPolicy             = NX_SUPPORT_POLICY_OPTIN;
     wcscpy( data->NtSystemRoot, L"C:\\windows" );
 
-    features = data->ProcessorFeatures;
-    switch (sci.ProcessorArchitecture)
-    {
-    case PROCESSOR_ARCHITECTURE_INTEL:
-    case PROCESSOR_ARCHITECTURE_AMD64:
-        features[PF_COMPARE_EXCHANGE_DOUBLE]              = !!(sci.ProcessorFeatureBits & CPU_FEATURE_CX8);
-        features[PF_MMX_INSTRUCTIONS_AVAILABLE]           = !!(sci.ProcessorFeatureBits & CPU_FEATURE_MMX);
-        features[PF_XMMI_INSTRUCTIONS_AVAILABLE]          = !!(sci.ProcessorFeatureBits & CPU_FEATURE_SSE);
-        features[PF_3DNOW_INSTRUCTIONS_AVAILABLE]         = !!(sci.ProcessorFeatureBits & CPU_FEATURE_3DNOW);
-        features[PF_RDTSC_INSTRUCTION_AVAILABLE]          = !!(sci.ProcessorFeatureBits & CPU_FEATURE_TSC);
-        features[PF_PAE_ENABLED]                          = !!(sci.ProcessorFeatureBits & CPU_FEATURE_PAE);
-        features[PF_XMMI64_INSTRUCTIONS_AVAILABLE]        = !!(sci.ProcessorFeatureBits & CPU_FEATURE_SSE2);
-        features[PF_SSE3_INSTRUCTIONS_AVAILABLE]          = !!(sci.ProcessorFeatureBits & CPU_FEATURE_SSE3);
-        features[PF_SSSE3_INSTRUCTIONS_AVAILABLE]         = !!(sci.ProcessorFeatureBits & CPU_FEATURE_SSSE3);
-        features[PF_XSAVE_ENABLED]                        = !!(sci.ProcessorFeatureBits & CPU_FEATURE_XSAVE);
-        features[PF_COMPARE_EXCHANGE128]                  = !!(sci.ProcessorFeatureBits & CPU_FEATURE_CX128);
-        features[PF_SSE_DAZ_MODE_AVAILABLE]               = !!(sci.ProcessorFeatureBits & CPU_FEATURE_DAZ);
-        features[PF_NX_ENABLED]                           = !!(sci.ProcessorFeatureBits & CPU_FEATURE_NX);
-        features[PF_SECOND_LEVEL_ADDRESS_TRANSLATION]     = !!(sci.ProcessorFeatureBits & CPU_FEATURE_2NDLEV);
-        features[PF_VIRT_FIRMWARE_ENABLED]                = !!(sci.ProcessorFeatureBits & CPU_FEATURE_VIRT);
-        features[PF_RDWRFSGSBASE_AVAILABLE]               = !!(sci.ProcessorFeatureBits & CPU_FEATURE_RDFS);
-        features[PF_FASTFAIL_AVAILABLE]                   = TRUE;
-        features[PF_SSE4_1_INSTRUCTIONS_AVAILABLE]        = !!(sci.ProcessorFeatureBits & CPU_FEATURE_SSE41);
-        features[PF_SSE4_2_INSTRUCTIONS_AVAILABLE]        = !!(sci.ProcessorFeatureBits & CPU_FEATURE_SSE42);
-        features[PF_AVX_INSTRUCTIONS_AVAILABLE]           = !!(sci.ProcessorFeatureBits & CPU_FEATURE_AVX);
-        features[PF_AVX2_INSTRUCTIONS_AVAILABLE]          = !!(sci.ProcessorFeatureBits & CPU_FEATURE_AVX2);
-        break;
-
-    case PROCESSOR_ARCHITECTURE_ARM:
-        features[PF_ARM_VFP_32_REGISTERS_AVAILABLE]       = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_VFP_32);
-        features[PF_ARM_NEON_INSTRUCTIONS_AVAILABLE]      = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_NEON);
-        break;
-
-    case PROCESSOR_ARCHITECTURE_ARM64:
-        features[PF_ARM_V8_INSTRUCTIONS_AVAILABLE]        = TRUE;
-        features[PF_ARM_V8_CRC32_INSTRUCTIONS_AVAILABLE]  = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_V8_CRC32);
-        features[PF_ARM_V8_CRYPTO_INSTRUCTIONS_AVAILABLE] = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_V8_CRYPTO);
-        features[PF_ARM_V81_ATOMIC_INSTRUCTIONS_AVAILABLE]= !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_V81_ATOMIC);
-        features[PF_ARM_V82_DP_INSTRUCTIONS_AVAILABLE]    = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_V82_DP);
-        features[PF_ARM_V83_JSCVT_INSTRUCTIONS_AVAILABLE] = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_V83_JSCVT);
-        features[PF_ARM_V83_LRCPC_INSTRUCTIONS_AVAILABLE] = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_V83_LRCPC);
-        features[PF_ARM_SVE_INSTRUCTIONS_AVAILABLE]       = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_SVE);
-        features[PF_ARM_SVE2_INSTRUCTIONS_AVAILABLE]      = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_SVE2);
-        features[PF_ARM_SVE2_1_INSTRUCTIONS_AVAILABLE]    = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_SVE2_1);
-        features[PF_ARM_SVE_AES_INSTRUCTIONS_AVAILABLE]   = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_SVE_AES);
-        features[PF_ARM_SVE_PMULL128_INSTRUCTIONS_AVAILABLE] = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_SVE_PMULL128);
-        features[PF_ARM_SVE_BITPERM_INSTRUCTIONS_AVAILABLE] = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_SVE_BITPERM);
-        features[PF_ARM_SVE_BF16_INSTRUCTIONS_AVAILABLE]  = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_SVE_BF16);
-        features[PF_ARM_SVE_EBF16_INSTRUCTIONS_AVAILABLE] = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_SVE_EBF16);
-        features[PF_ARM_SVE_B16B16_INSTRUCTIONS_AVAILABLE] = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_SVE_B16B16);
-        features[PF_ARM_SVE_SHA3_INSTRUCTIONS_AVAILABLE]  = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_SVE_SHA3);
-        features[PF_ARM_SVE_SM4_INSTRUCTIONS_AVAILABLE]   = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_SVE_SM4);
-        features[PF_ARM_SVE_I8MM_INSTRUCTIONS_AVAILABLE]  = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_SVE_I8MM);
-        features[PF_ARM_SVE_F32MM_INSTRUCTIONS_AVAILABLE] = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_SVE_F32MM);
-        features[PF_ARM_SVE_F64MM_INSTRUCTIONS_AVAILABLE] = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_SVE_F64MM);
-        features[PF_COMPARE_EXCHANGE_DOUBLE]              = TRUE;
-        features[PF_NX_ENABLED]                           = TRUE;
-        features[PF_FASTFAIL_AVAILABLE]                   = TRUE;
-        /* add features for other architectures supported by wow64 */
-        for (i = 0; machines[i].Machine; i++)
-        {
-            switch (machines[i].Machine)
-            {
-            case IMAGE_FILE_MACHINE_ARMNT:
-                features[PF_ARM_VFP_32_REGISTERS_AVAILABLE]  = TRUE;
-                features[PF_ARM_NEON_INSTRUCTIONS_AVAILABLE] = TRUE;
-                break;
-            case IMAGE_FILE_MACHINE_I386:
-                features[PF_MMX_INSTRUCTIONS_AVAILABLE]    = TRUE;
-                features[PF_XMMI_INSTRUCTIONS_AVAILABLE]   = TRUE;
-                features[PF_RDTSC_INSTRUCTION_AVAILABLE]   = TRUE;
-                features[PF_XMMI64_INSTRUCTIONS_AVAILABLE] = TRUE;
-                features[PF_SSE3_INSTRUCTIONS_AVAILABLE]   = TRUE;
-                features[PF_RDTSCP_INSTRUCTION_AVAILABLE]  = TRUE;
-                features[PF_SSSE3_INSTRUCTIONS_AVAILABLE]  = TRUE;
-                features[PF_SSE4_1_INSTRUCTIONS_AVAILABLE] = TRUE;
-                features[PF_SSE4_2_INSTRUCTIONS_AVAILABLE] = TRUE;
-                break;
-            }
-        }
-        break;
-    }
-    data->ActiveProcessorCount = NtCurrentTeb()->Peb->NumberOfProcessors;
-    data->ActiveGroupCount = 1;
-
-    initialize_xstate_features( data );
-
     UnmapViewOfFile( data );
 }
 
-#include "pshpack1.h"
+#pragma pack(push,1)
 struct smbios_prologue
 {
     BYTE  calling_method;
@@ -642,7 +474,7 @@ struct smbios_wine_core_id_regs_arm64
     } regs[];
 };
 
-#include "poppack.h"
+#pragma pack(pop)
 
 #define RSMB (('R' << 24) | ('S' << 16) | ('M' << 8) | 'B')
 
