@@ -42,6 +42,7 @@ generic_handle_list_t generic_handle_list = LIST_INIT(generic_handle_list);
 
 static void write_type_v(FILE *f, const decl_spec_t *t, int is_field, bool define, const char *name, enum name_type name_type);
 
+static void write_apicontract( FILE *header, type_t *type );
 static void write_apicontract_guard_start(FILE *header, const expr_t *expr);
 static void write_apicontract_guard_end(FILE *header, const expr_t *expr);
 
@@ -559,11 +560,17 @@ static void write_type_v(FILE *h, const decl_spec_t *ds, int is_field, bool defi
 static void write_type_definition(FILE *f, type_t *t, bool define)
 {
     int in_namespace = t->namespace && !is_global_namespace(t->namespace);
-    int save_written = t->written;
     decl_spec_t ds = {.type = t};
     expr_t *contract = get_attrp(t->attrs, ATTR_CONTRACT);
 
+    if (t->written) return;
+
     if (contract) write_apicontract_guard_start(f, contract);
+    if (winrt_mode && define && type_get_type( t ) == TYPE_ENUM)
+    {
+        fprintf( f, "#ifndef __%s_ENUM_DEFINED__\n", t->c_name );
+        fprintf( f, "#define __%s_ENUM_DEFINED__\n", t->c_name );
+    }
     if(in_namespace) {
         fprintf(f, "#ifdef __cplusplus\n");
         fprintf(f, "} /* extern \"C\" */\n");
@@ -573,7 +580,7 @@ static void write_type_definition(FILE *f, type_t *t, bool define)
     write_type_left(f, &ds, NAME_DEFAULT, define, TRUE);
     fprintf(f, ";\n");
     if(in_namespace) {
-        t->written = save_written;
+        t->written = false;
         write_namespace_end(f, t->namespace);
         fprintf(f, "extern \"C\" {\n");
         fprintf(f, "#else\n");
@@ -582,6 +589,8 @@ static void write_type_definition(FILE *f, type_t *t, bool define)
         if (winrt_mode) write_widl_using_macros(f, t);
         fprintf(f, "#endif\n\n");
     }
+    if (winrt_mode && define && type_get_type( t ) == TYPE_ENUM)
+        fprintf( f, "#endif /* __%s_ENUM_DEFINED__ */\n", t->c_name );
     if (contract) write_apicontract_guard_end(f, contract);
 }
 
@@ -816,6 +825,8 @@ static void write_generic_handle_routines(FILE *header)
 static void write_typedef(FILE *header, type_t *type, bool define)
 {
     type_t *t = type_alias_get_aliasee_type(type), *root = type_pointer_get_root_type(t);
+    if (winrt_mode && !define && type_get_type( t ) == TYPE_ENUM)
+        write_type_definition( header, t, TRUE );
     if (winrt_mode && root->namespace && !is_global_namespace(root->namespace))
     {
         fprintf(header, "#ifndef __cplusplus\n");
@@ -1254,7 +1265,7 @@ static void write_inline_wrappers(FILE *header, const type_t *iface, const type_
     if (!is_callas(func->attrs)) {
       const var_t *arg;
 
-      fprintf(header, "static __WIDL_INLINE ");
+      fprintf(header, "static inline ");
       write_type_decl_left(header, type_function_get_ret(func->declspec.type));
       fprintf(header, " %s_%s(", name, get_name(func));
       write_args(header, type_function_get_args(func->declspec.type), name, 1, FALSE, NAME_C);
@@ -1574,11 +1585,12 @@ static char *format_apicontract_macro(const type_t *type)
 
 static void write_apicontract_guard_start(FILE *header, const expr_t *expr)
 {
-    const type_t *type;
+    type_t *type;
     char *name;
     int ver;
     if (!winrt_mode) return;
-    type = expr->u.tref.type;
+    type = expr->u.var->declspec.type;
+    write_apicontract( header, type );
     ver = expr->ref->u.integer.value;
     name = format_apicontract_macro(type);
     fprintf(header, "#if %s_VERSION >= %#x\n", name, ver);
@@ -1591,7 +1603,7 @@ static void write_apicontract_guard_end(FILE *header, const expr_t *expr)
     char *name;
     int ver;
     if (!winrt_mode) return;
-    type = expr->u.tref.type;
+    type = expr->u.var->declspec.type;
     ver = expr->ref->u.integer.value;
     name = format_apicontract_macro(type);
     fprintf(header, "#endif /* %s_VERSION >= %#x */\n", name, ver);
@@ -1823,11 +1835,14 @@ static void write_coclass_forward(FILE *header, type_t *cocl)
 
 static void write_apicontract(FILE *header, type_t *apicontract)
 {
-    char *name = format_apicontract_macro(apicontract);
+    char *name;
+    if (apicontract->written) return;
+    name = format_apicontract_macro( apicontract );
     fprintf(header, "#if !defined(%s_VERSION)\n", name);
     fprintf(header, "#define %s_VERSION %#x\n", name, get_attrv(apicontract->attrs, ATTR_CONTRACTVERSION));
     fprintf(header, "#endif // defined(%s_VERSION)\n\n", name);
     free(name);
+    apicontract->written = true;
 }
 
 static void write_runtimeclass(FILE *header, type_t *runtimeclass)
@@ -1962,6 +1977,26 @@ static void write_forward_decls(FILE *header, const statement_list_t *stmts)
   }
 }
 
+static void write_header_stmts( FILE *header, const statement_list_t *stmts, const type_t *iface, int ignore_funcs );
+
+static void write_header_com_interface( FILE *header, type_t *iface, const type_t *ref_iface )
+{
+    type_t *inherit_from;
+
+    if (iface->written) return;
+
+    /* ensure declaration of inherited interface exists before ours (C++ requires this) */
+    inherit_from = type_iface_get_inherit( iface );
+    if (inherit_from && !inherit_from->ignore)
+        write_header_com_interface( header, inherit_from, inherit_from );
+
+    write_com_interface_start( header, iface );
+    write_header_stmts( header, type_iface_get_stmts(iface), ref_iface, TRUE );
+    write_com_interface_end( header, iface );
+
+    iface->written = true;
+}
+
 static void write_header_stmts(FILE *header, const statement_list_t *stmts, const type_t *iface, int ignore_funcs)
 {
   const statement_t *stmt;
@@ -1978,9 +2013,7 @@ static void write_header_stmts(FILE *header, const statement_list_t *stmts, cons
           if (is_object(iface)) is_object_interface++;
           if (is_attr(stmt->u.type->attrs, ATTR_DISPINTERFACE) || is_object(stmt->u.type))
           {
-            write_com_interface_start(header, iface);
-            write_header_stmts(header, type_iface_get_stmts(iface), stmt->u.type, TRUE);
-            write_com_interface_end(header, iface);
+            write_header_com_interface(header, iface, stmt->u.type);
             if (async_iface)
             {
               write_com_interface_start(header, async_iface);
@@ -2013,11 +2046,10 @@ static void write_header_stmts(FILE *header, const statement_list_t *stmts, cons
       case STMT_TYPEREF:
         /* FIXME: shouldn't write out forward declarations for undefined
         * interfaces but a number of our IDL files depend on this */
-        if (type_get_type(stmt->u.type) == TYPE_INTERFACE && !stmt->u.type->written)
+        if (type_get_type(stmt->u.type) == TYPE_INTERFACE)
           write_forward(header, stmt->u.type);
         break;
       case STMT_IMPORTLIB:
-      case STMT_PRAGMA:
         /* not included in header */
         break;
       case STMT_IMPORT:
@@ -2043,6 +2075,9 @@ static void write_header_stmts(FILE *header, const statement_list_t *stmts, cons
         write_header_stmts(header, stmt->u.type->details.module->stmts, stmt->u.type, FALSE);
         fprintf(header, "#endif /* __%s_MODULE_DEFINED__ */\n", stmt->u.type->name);
         break;
+      case STMT_PRAGMA:
+        if (!strncmp( stmt->u.str, "pack", 4 )) fprintf(header, "#pragma %s\n", stmt->u.str);
+        break;
       case STMT_CPPQUOTE:
         fprintf(header, "%s\n", stmt->u.str);
         break;
@@ -2099,14 +2134,6 @@ void write_header(const statement_list_t *stmts)
   fprintf(header, "#ifndef __%s__\n", header_token);
   fprintf(header, "#define __%s__\n\n", header_token);
 
-  fprintf(header, "#ifndef __WIDL_INLINE\n");
-  fprintf(header, "#if defined(__cplusplus) || defined(_MSC_VER)\n");
-  fprintf(header, "#define __WIDL_INLINE inline\n");
-  fprintf(header, "#elif defined(__GNUC__)\n");
-  fprintf(header, "#define __WIDL_INLINE __inline__\n");
-  fprintf(header, "#endif\n");
-  fprintf(header, "#endif\n\n");
-
   fprintf(header, "/* Forward declarations */\n\n");
   write_forward_decls(header, stmts);
 
